        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         wspeed.asm
; Function:     These are the wind speed support routines.
; Routines:     calwsp: Find/Store wind speed sensor ID.
;               inispd: Initialize wind speed measurements
;               newspd: re-initialize variables after 5-minute roll
;               iqwspd: Read Wind speed data
;               spdstt: Do wind speed stats
;               piwind: Convert wind speed to ASCII
;               pmwind: Convert wind speed to ASCII (mainline)
;               pnwind: Convert wind speed to ASCII (mainline metric)
;               avgspd: Compute 5-minute average speed
;               ldxams: Load XA with 10-minute max speed (max of maxws & maxw2)
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      calwsp
; Called By:    ssens
; Calls:        isrch (find sensor ID)
;               wrmess (write 2-line message to LCD)
;               wr4mes (wrire 4-line message to LCD)
;               wfnkey (wait for next key)
;               cnvid (convert ID to ASCII)
;               wrstr (write string to LCD)
;               wrcmd (write LCD command byte)
;               wrdat (write LCD data byte
;               rdf64 (read flash 64-byte block)
;               wrf64 (write flash 64-byte block)
;               appera (erase flash block)
; Gazintas:     None
; Gazoutas:     Flash location ficntr is programmed
; Function:     This is the routine to calibrate the wind direction sensors.
;               This is a pretty nasty chore as we do not know ahead of time
;               anything about the eight direction sensors.  The first step is
;               to find out the IDs of the eight sensors.  We do this by
;               asking the user to rotate the wind vane clockwise
;               (incrementing degrees).  When the user starts the rotations,
;               we first try and find eight unique serial numbers.  If we get
;               any repeats before these eight unique numbers, we assume the
;               user screwed up and start over again.  While we are getting
;               these new IDs, we show the numbers 1-8 on the display along
;               with the text 1/3 indicating we are working on step 1 of three.
;               Step two happens after we get the eighth (and last)consecutive
;               new ID.  We change the counter to show 2/3 and expect to see
;               all eight IDs we got in step 1 to happen again in sequence.
;               If this fails, we go back to step 1.  Step three is the same
;               as step two, is probably not necessary, but I want to do it
;               anyway.
;
;               The final step is to tell the person to point the weather
;               vane north (as if the wind was coming from the north), and
;               press select.  This establishes the internal offset so we know
;               what direction is north.
;
; -----------------------------------------------------------------------------

;       Finding the wind speed sensor gets more complicated with the addition
;       of the rain guage as we have two sensors with the same ID.  First I
;       see if there is more than one "1D" ID.  If there is only one ID, it
;       must be the wind speed, so we don't ask any questions and program the
;       wind ID with the found value.

calwsp: lda     ficntr          ; get the wind speed counter family ID
        cmp     #$30            ; see if 30 (means TAI603 already programmed)
        bne     calwsd          ; if not, normal processing

        lda     #54             ; Use TAI603? message
        jsr     wrmess          ; write it
calw6p: jsr     wfnkey          ; and wait for a key
        cmp     #kup            ; see if up (TAI603)
        beq     calwsx          ; if Yes, no action required, exit
        cmp     #kdown          ; see if down (not TAI603)
        bne     calw6p          ; if not up or down, go try again

;       If they chose not to use the TAI603, look for other sensors...

calwsd: lda     #$1d            ; family code for the DS2423 counter
        jsr     i2srch          ; go see if there are any
        tsta                    ; set flags on A (just in case)
        beq     calwns          ; if none, error
        deca                    ; see if only one sensor
        beq     calws1          ; go there if so
        bra     calws2          ; if 2, got some decisions to make

calwns: lda     #46             ; No Sensor message
        jsr     wrmess          ; write it
        lda     #40             ; 4 seconds
        jsr     waits           ; wait it
calwsx: rts                     ; and exit


; We get here if there are two counters on the 1-wire bus.  I need some help
; from the user to figger which is which.  Two choices are offered.  The first
; is to have the user unplug the rain guage and then I'll know the remaining
; guage is the wind speed sensor (easiest if the sensor can be removed).
; Second option is if the user knows which is which by the ID, I just display
; the IDs and let the user pick the ID that is the correct one for the wind
; speed sensor.  In either case, when I do the rain guage, it is by definition
; the sensor that is NOT the wind speed sensor.


calws2: lda     #9              ; big message #9 (resolution choices)
        jsr     wr4mes          ; write it

ssq2:   jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; see if up key (remove a sensor)
        beq     calwsp          ; if so, to see if one sensor now
        cmp     #kdown          ; see if down button (user pick)
        beq     ssqry           ; if so, check IDs again before cal
        bra     ssq2            ; if neither, ignore.

;       We get here if the user chose to look at both IDs and decide which is
;       the real wind sensor.  This is now a shared routine.

ssqry:  jsr     i2pick          ; let user pick, answer in (mndray)

;       if only one ID or the user picked the one they wanted, we get here to
;       program the ID.
;       To update the flash block that has other addresses we don't want to
;       mess with we copy the entire block to (msg), modify the wind speed
;       address, then write the entire block back.
calws1: ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

        ldhx    mndray          ; get bytes 0,1
        sthx    msg+16          ; store them
        ldhx    mndray+2        ; get bytes 0,1
        sthx    msg+18          ; store them
        ldhx    mndray+4        ; get bytes 0,1
        sthx    msg+20          ; store them
        ldhx    mndray+6        ; get bytes 0,1
        sthx    msg+22          ; store them

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values

        cli                     ; interrupts back on
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      inispd, newspd
; Called By:    dowx,timrtn
; Calls:        None
; Gazintas:     None
; Gazoutas:     lastws (wind speed counter last sample)
;               cumws2 (totalized wind speed ticks)
;               errspd (wind speed error flag/counter)
;               wsamp (wind speed sample counter)
; Function:     This is where we initialize wind speed measurement stats.
;               The maximum wind speed gets cleared to 0 as well as the
;               cumulative wind speed samples.
;               Wind speed is measured by looking at the difference in counts
;               between two readings 5 seconds apart.  Since we are just
;               starting, we don't know the previous reading.  I solve this by
;               setting the error flag to say the previous reading was invalid
;               so the next reading will not mess up the stats.  Note that
;               setting error flags and incrementing the error counter are
;               seperate functions and this does not increment the error
;               counter.
;               For the 5-minute roll, all I need to clear are the cumulative
;               wind speed stats and the max wind speed, so I do a little code
;               sharing here.
; -----------------------------------------------------------------------------
inispd:
        clr     lastws          ; clear instant wind speed
        clr     lastws+1        ; All bytes
        clr     lastws+2        ; All bytes
        clr     lastws+3        ; All bytes

        bset    7,errspd        ; Set "previous error" flag
        bset    6,errspd        ; set "current error" flag too

        clra                    ; 0...
        sta     ptavws          ; Peet Mode Average Wind speed MSB init to 0
        sta     ptavws+1        ; Do LSB too

newspd: lda     maxws           ; get current max wind speed (MSB)
        sta     maxw2           ; save it as previous
        lda     maxws+1         ; get current max wind speed (LSB)
        sta     maxw2+1         ; save it as previous

        clr     maxws           ; clear max wind speed
        clr     maxws+1         ; LSB too
        clr     cumws2          ; clear average wind speed
        clr     cumws2+1        ; LSB too
        clr     wsamp           ; clear wind speed sample counter

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqwspd
; Called By:    timrtn
; Calls:        sndadr (send 1-wire address)
;               i_writ (write 1-wire byte with CRC)
;               i_read (read 1-wire byte with CRC)
;               crcini (initialize CRC engin)
;               crcchk (check CRC result)
; Gazintas:     errspd (error counter/flag)
; Gazoutas:     winspd (wind speed in ticks)
; Internals:    rdcnt (wind speed counter absolute value)
;               lastws (last wind speed counter value)
; Function:     This is the routine to read the wind speed.  The anemometer
;               cups are hooked up to magnets that wiggle a reed switch.  The
;               DS2423 ibutton IC has two 32-bit counters on it that can be
;               hooked up to an external trigger.  These counters cannot be
;               reset, so we need to take two readings and figger the
;               difference.  We do this by taking successive readings of the
;               counter value at 5-second intervals and compute the difference
;               between the two readings to get the wind speed in ticks.  The
;               ticks get converted to MPH later.  Whil getting all this data,
;               this routine also tracks the CRC values of all bytes read and
;               writtend and confirms them against the data from the DS2423.
;               Errors are noted and flagged.  See the comment in ram.asm for
;               how the error flags work.
; -----------------------------------------------------------------------------
iqwspd: lda     ficntr          ; get the wind speed counter family ID
        cmp     #$30            ; see if 30 (means TAI603)
        bne     iqwsnn          ; if not, normal processing

;       If the ID is $30, then we have the new TAI603 sensor, go to it's
;       code for temperature readings.
        jmp     t3rwsp          ; go there, it'll RTS back to the caller

iqwsnn: lda     errspd          ; get error flags
        and     #$c0            ; see if any errors in previous data
        beq     iqwsgd          ; if not, no error flag processing needed

        lda     errspd          ; get error counter and flags
        brset   6,errspd,rwine1 ; if first error, clear only 1 bit
        and     #$3f            ; otherwise clear both flags
rwine1: and     #$bf            ; clear first error flag
        sta     errspd          ; save new set of error flags

iqwsgd: ldhx    #ficntr         ; address for counter
        jsr     sndadr          ; send it off
        jsr     crcini          ; reset the 16-bit CRC here
        lda     #$a5            ; read memory+counter command
        jsr     i_writ          ; send it
        lda     #$df            ; memory address LSB
        jsr     i_writ          ; send it
        lda     #$01            ; memory address MSB
        jsr     i_writ          ; send it
        jsr     i_read          ; read back memory byte (& toss)
        jsr     i_read          ; counter byte 0 (LSB)
        sta     rdcnt1+3        ; store it
        jsr     i_read          ; counter byte 1
        sta     rdcnt1+2        ; store it
        jsr     i_read          ; counter byte 2
        sta     rdcnt1+1        ; store it
        jsr     i_read          ; counter byte 3 (MSB)
        sta     rdcnt1+0        ; store it
        jsr     i_read          ; dummy byte 1 (ignored)
        jsr     i_read          ; dummy byte 2 (ignored)
        jsr     i_read          ; dummy byte 3 (ignored)
        jsr     i_read          ; dummy byte 4 (ignored)
        jsr     i_read          ; CRC byte 1
        jsr     i_read          ; CRC byte 2
        jsr     crcchk          ; see if we get the magic answer
        bne     iqsbd           ; If error, flag it otherwise good

; before we really declare the data good I need to check that the current
; wind count value is greater than the previous wind count value.  An
; error that shows up is that the wind counter gets reset for some reason
; and I get two legit values, but the difference is a big nasty, illegal
; value that usually ends up being 99MPH.  Note that once the data is
; verified as good, I only need to do a 16-bit delta reading.

iqsgd:  lda     rdcnt1+0        ; get current reading MSB
        cmp     lastws+0        ; compare to last reading
        bhi     iqsdm           ; if greater, legit reading
        blo     iqsbd1          ; if less, error!
        lda     rdcnt1+1        ; get current reading next byte
        cmp     lastws+1        ; compare to last reading
        bhi     iqsdm           ; if greater, legit reading
        blo     iqsbd1          ; if less, error!
        lda     rdcnt1+2        ; get current reading next byte
        cmp     lastws+2        ; compare to last reading
        bhi     iqsdm           ; if greater, legit reading
        blo     iqsbd1          ; if less, error!
        lda     rdcnt1+3        ; get current reading next byte
        cmp     lastws+3        ; compare to last reading
        blo     iqsbd1          ; if less, error!
        bra     iqsdm           ; otherwise good

*       If there is a rollover error, go ahead and store the current value as
*       the last known good, even though it may not be.  We still need
*       two good readings in a row before we declare good data.
iqsbd1: lda     rdcnt1+3        ; get current reading
        sta     lastws+3        ; make it the "last" reading for next time
        lda     rdcnt1+2        ; get current reading
        sta     lastws+2        ; make it the "last" reading for next time
        lda     rdcnt1+1        ; get current reading
        sta     lastws+1        ; make it the "last" reading for next time
        lda     rdcnt1+0        ; get current reading
        sta     lastws+0        ; make it the "last" reading for next time

iqsbd:  lda     errspd          ; get wind speed error counter
        inca                    ; increment wind speed error counter value
        ora     #$c0            ; set error-now flags
        sta     errspd          ; save counter and flag
        clr     winspd+1        ; clear wind speed
        clr     winspd+0        ; clera wind speed
        bra     iqwnx           ; and exit




iqsdm:  lda     rdcnt1+3        ; get current count LSB
        sub     lastws+3        ; subtract previous reading
        sta     winspd+1        ; save result

        lda     rdcnt1+2        ; get current reading MSB
        sbc     lastws+2        ; subtract previous reading
        sta     winspd+0        ; save result

; If I see a speed above 25MPH, send debug data TEST CODE ONLY
        lda     winspd          ; get wind speed MSB
        bne     iqwdbg          ; if not zero, we got problems
        lda     winspd+1        ; get wind speed LSB
        cmp     #100            ; value for roughly 25MPH
        blo     iqwndb          ; if less, go as normal

iqwdbg: lda     #3              ; wind speed data flag
        sta     dbbuf+0         ; save it

        lda     winspd+0        ; wind speed MSB
        sta     dbbuf+1         ; save it
        lda     winspd+1        ; wind speed LSB
        sta     dbbuf+2         ; save it

        lda     rdcnt1+2        ; read current count MSB
        sta     dbbuf+3         ; save it
        lda     rdcnt1+3        ; read current count MSB
        sta     dbbuf+4         ; save it

        lda     lastws+2        ; read last count MSB
        sta     dbbuf+5         ; save it
        lda     lastws+3        ; read last count MSB
        sta     dbbuf+6         ; save it

        lda     #0              ; 0...
        sta     dbbuf+7         ; set to 0, will get filled in later

        lda     #8              ; eight debug bytes
        sta     dbptr           ; flag for next transmission
; End of debug code

iqwndb: lda     rdcnt1+3        ; get current reading
        sta     lastws+3        ; make it the "last" reading for next time
        lda     rdcnt1+2        ; get current reading
        sta     lastws+2        ; make it the "last" reading for next time
        lda     rdcnt1+1        ; get current reading
        sta     lastws+1        ; make it the "last" reading for next time
        lda     rdcnt1+0        ; get current reading
        sta     lastws+0        ; make it the "last" reading for next time

iqwnx:  rts                     ; all done sampling

        page
; -----------------------------------------------------------------------------
; Routine:      spdstt
; Called By:    timrtn
; Calls:        None
; Gazintas:     winspd (current sample wind speed in ticks)
;               errspd (error flags/counter)
; Gazoutas:     cumws2 (5-minut cumulative wind speed)
;               wsamp (sample count)
;               maxws (max wind speed in ticks)
; Function:     This is the statistics gathering code.  We check the current
;               wind speed value (assuming it was a good sample) against the
;               current max speed as well as add its value to the totalizer so
;               we can determine the 5-minute average wind seed.  We also
;               increment the wind speed sample counter if the sample was
;               good so we know what number to divide by for the 5-minute
;               average.
;               If this is the 59th sample in the 5-minute count we also
;               calculate the average wind speed for this past 5-minute cycle
;               and store it in the ptavws for Peet modes.  Since I need to
;               store it, I also retrieve that value for the TNC message too.
; -----------------------------------------------------------------------------
spdstt: lda     errspd          ; get error flags
        and     #$c0            ; see if any errors in current or previous data
        bne     rwinav          ; if any bad data, don't process in stats

        lda     winspd+1        ; get current count LSB
        add     cumws2+1        ; add previous reading (for 5-minute)
        sta     cumws2+1        ; save result
        lda     winspd+0        ; get current count MSB
        adc     cumws2+0        ; add previous reading
        sta     cumws2+0        ; save result
        inc     wsamp           ; increment wind sample count

;       Do the 24-hour gust stats...
spd24h: lda     winspd          ; get current wind speed MSB
        cmp     todygs          ; compare to gust
        blo     spd5m           ; if MSBs less, then not new max
        bhi     rwnmxx          ; if greater, then new max

        lda     winspd+1        ; get current wind speed LSB
        cmp     todygs+1        ; compare to gust
        bls     spd5m           ; if LSBs less or same, then not new max

rwnmxx: lda     winspd          ; get wind speed MSB
        sta     todygs          ; store it as new max
        lda     winspd+1        ; get wind speed LSB
        sta     todygs+1        ; store as new max

;       Do the 5-minute gust stats...
spd5m:  lda     winspd          ; get current wind speed MSB
        cmp     maxws           ; compare to gust
        blo     rwinav          ; if MSBs less, then not new max
        bhi     rwnmax          ; if greater, then new max

        lda     winspd+1        ; get current wind speed LSB
        cmp     maxws+1         ; compare to gust
        bls     rwinav          ; if LSBs less or same, then not new max

rwnmax: lda     winspd          ; get wind speed MSB
        sta     maxws           ; store it as new max
        lda     winspd+1        ; get wind speed LSB
        sta     maxws+1         ; store as new max

rwinav: lda     cumcnt          ; get the current 5-minute cycle counter
        cmp     #59             ; see if the last sample in the cycle
        bne     rwinex          ; if not last, go exit normally.

        lda     cumws2          ; get cumulative speed MSB
        sta     divdnd          ; store for division
        lda     cumws2+1        ; get cumulative speed LSB
        sta     divdnd+1        ; store it for division
        lda     wsamp           ; the divisor (total good samples)
        beq     rwina0          ; if no samples, problems!
        sta     divsor          ; set it up

        jsr     divu            ; do the division

        lda     quotnt          ; get quotient MSB
        sta     ptavws          ; store for conversion
        lda     quotnt+1        ; get quotient LSB
        sta     ptavws+1        ; store it
        bra     rwinex          ; and done

rwina0:clra                     ; If no samples for some reason, just say 0
        sta     ptavws          ; store MSB
        sta     ptavws+1        ; store LSB

rwinex: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      piwind
; Called By:    tncmsg, cmpmsg
; Calls:        bin2as (convert binary to ASCII)
; Gazintas:     cnv2a (wind speed in ticks)
; Gazoutas:     atemp (ASCII wind speed in MPH)
; Function:     This is the routine to convert wind ticks to wind speed.
;               After much guesswork and finally a calibrated wind tunnel,
;               Russ got the magic number below.  This represents the number
;               of miles per hour per tick when the sample period is five
;               seconds.
;
;               The current constant is 0.227806 miles per hour per tick.
;               The closest in binary is 233/1024ths or $e9 in 0.10 format.
;
;               The max displayable speed is 99MPH which translates to a
;               reading of 399 ticks.  Since the multiply operation is only
;               8-bit, I check for a reading above 255 and then add the 256
;               count offset (63.5MPH) to the mod-256 conversion of the
;               remainder.  In the unlikely event that the wind speed tops
;               99MPH, we just "peg" at 99MPH for the speed.
; -----------------------------------------------------------------------------
piwind: clr     temp            ; clear wind speed offset
        lda     cnv2a+0         ; Get speed MSB
        beq     piwmod          ; if 0, no special processing
        cmp     #1              ; see if MSB is 1
        bne     pwibig          ; if >1, show 99 for speed

        lda     #127            ; 63.5 in 7.1 format (256 ticks in MPH)
        sta     temp            ; save for later

piwmod: lda     cnv2a+1         ; get wind speed counter LSB only
        ldx     #$e9            ; our constant (times four)
        mul                     ; multiply the two, answer is x:a in 6:10
        lsrx                    ; divide by two to get 7:9
        rora                    ; complete division.

        psha                    ; save a for a bit
        txa                     ; get MSB in a
        add     temp            ; add offset (if any)
        tax                     ; put MSB back in x
        pula                    ; get A back

        lsrx                    ; divide by two again to get 8:8
        rora                    ; complete division.

        bpl     pwirnd          ; if fraction <0.5, don't round
        incx                    ; otherwise round up
pwirnd: stx     divdnd+1        ; divdnd will be MSB of value to convert
        clr     divdnd          ; make MSB 0
        jsr     bin2as          ; go convert it
        rts                     ; all done.

pwibig: lda     #'9'            ; just show max possible
        sta     atemp+1         ; tens
        sta     atemp+2         ; units
        rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      pmwind
; Called By:    showit
; Calls:        bin2am (convert binary to ASCII)
; Gazintas:     macnv (wind speed in ticks)
; Gazoutas:     matemp (ASCII wind speed in MPH)
; Function:     This is the mainline version of piwind (no variable conflicts)
;               See piwind for theory of operation
; -----------------------------------------------------------------------------
pmwind: clr     matemp          ; clear wind speed offset
        lda     macnv+0         ; Get speed MSB
        beq     pmwmod          ; if 0, no special processing
        cmp     #1              ; see if MSB is 1
        bne     pmibig          ; if >1, show 99 for speed

        lda     #127            ; 63.5 in 7.1 format (256 ticks in MPH)
        sta     matemp          ; save for later

pmwmod: lda     macnv+1         ; get wind speed counter LSB only
        ldx     #$e9            ; our constant (times four)
        mul                     ; multiply the two, answer is x:a in 6:10
        lsrx                    ; divide by two to get 7:9
        rora                    ; complete division.

        psha                    ; save a for a bit
        txa                     ; get MSB in a
        add     matemp          ; add offset (if any)
        tax                     ; put MSB back in x
        pula                    ; get A back

        lsrx                    ; divide by two again to get 8:8
        rora                    ; complete division.

        bpl     pmirnd          ; if fraction <0.5, don't round
        incx                    ; otherwise round up
pmirnd: stx     mbcnv+1         ; divdnd will be MSB of value to convert
        clr     mbcnv           ; make MSB 0
        jsr     bin2am          ; go convert it
        rts                     ; all done.

pmibig: lda     #'9'            ; just show max possible
        sta     matemp+1        ; tens
        sta     matemp+2        ; units
        rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      pnwind
; Called By:    showit
; Calls:        bin2am (convert binary to ASCII)
; Gazintas:     macnv (wind speed in ticks)
; Gazoutas:     matemp (ASCII wind speed in MPH)
; Function:     This is the metric mainline version of piwind (no variable
;               conflicts)  See piwind for theory of operation.
;               For Km/h, the magic constant is:
;                0.366618 Km/h/tick
;                188/512ths
;                $bc 512ths
; -----------------------------------------------------------------------------
pnwind: clr     matemp          ; clear wind speed offset
        lda     macnv+0         ; Get speed MSB
        beq     pnwmod          ; if 0, no special processing
        cmp     #1              ; see if MSB is 1
        bne     pnibig          ; if >1, show 99 for speed

        lda     #$bc            ; 93.854 in 7.1 format (256 ticks in KPH)
        sta     matemp          ; save for later

pnwmod: lda     macnv+1         ; get wind speed counter LSB only
        ldx     #$bc            ; our constant (times four)
        mul                     ; multiply the two, answer is x:a in 7:9

        psha                    ; save a for a bit
        txa                     ; get MSB in a
        add     matemp          ; add offset (if any)
        tax                     ; put MSB back in x
        pula                    ; get A back

        lsrx                    ; divide by two again to get 8:8
        rora                    ; complete division.

        bpl     pnirnd          ; if fraction <0.5, don't round
        incx                    ; otherwise round up
pnirnd: stx     mbcnv+1         ; divdnd will be MSB of value to convert
        clr     mbcnv           ; make MSB 0
        jsr     bin2am          ; go convert it
        rts                     ; all done.

pnibig: lda     #'9'            ; just show max possible
        sta     matemp+1        ; tens
        sta     matemp+2        ; units
        rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      ldxams
; Called By:    tncmsg
; Calls:        None
; Gazintas:     None (maxws and maxw2 are "globals"
; Gazoutas:     X:A higher of maxws or maxw2
; Function:     This routine is used to get the highest gust over the past 10
;               minutes.  MAXWS is the highest over the past 5 minutes and
;               MAXW2 is the value of MAXWS from the previous 5 minutes.
;               Russ says NOAA says this needs to be a 10-minute value.  For
;               now I'm only affecting broadcast data, the display will still
;               show the 5-minute gust value.
; -----------------------------------------------------------------------------
ldxams: lda     maxw2           ; get older max ws MSB
        cmp     maxws           ; compare to current max
        blo     ldxanu          ; newer one is highest
        bhi     ldxaol          ; older one is highest

        ; if MSBs the same, compare LSBs
        lda     maxw2+1         ; get older max ws MSB
        cmp     maxws+1         ; compare to current max
        blo     ldxanu          ; newer one is highest

ldxaol: ldx     maxw2           ; oldest is highest, X=MSB
        lda     maxw2+1         ; oldest is highest, A=LSB
        bra     ldxaex          ; done

ldxanu: ldx     maxws           ; newest is highest, X=MSB
        lda     maxws+1         ; newest is highest, A=LSB
ldxaex: rts                     ; done

