.pagewidth 132
.header "APRS Weather station Controller"
.subheader "Ver 1.16.1R April 11, 2009"
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
* This is the control program for an MC68HC908GP32 that will read data from a
* Dallas Semi weather sensor and both display it on an LCD and send it to a
* TNC for APRS weather reporting.
*
* MC68HC906GP32 port assignments
* PA0 (I/O) debug   PB0 (O) 1-wire I/O PC0 (O)   Radio Ctl  PD0 (O) MX614 M0
* PA1 (I/O) LCD RS  PB1 (O) 1-wire Pwr PC1 (O)   Unused     PD1 (O) MX614 TXD
* PA2 (I/O) LCD RW  PB2 (O) 1-wire Pwr PC2 (O)   Unused     PD2 (I) MX614 RXD
* PA3 (I/O) LCD STB PB3 (I) Up SW      PC3 (O)   Unused     PD3 (O) MX614 CLK
* PA4 (I/O) LCD D4  PB4 (I) Down SW    PC4 (O)   MX614 DET  PD4 (O) T1CH0 (LED)
* PA5 (I/O) LCD D5  PB5 (I) Select SW  PC5 N/A   N/A        PD5 (I) T1CH1 (/RDY)
* PA6 (I/O) LCD D6  PB6 (I) Menu SW    PC6 N/A   N/A        PD6 N/A N/A
* PA7 (I/O) LCD D6  PB7 (0) Pwr Sense  PC7 N/A   N/A        PD7 N/A N/A
*
* OVERVIEW
* ========
* This program talks to a demo weather sensor kit supplied by Dallas Semi-
* conductor.  The sensors are hooked up to their 1-wire interface.  This
* program reads the raw data from those sensors (pulse counts, sensor serial
* numbers, etc.), converts it to something rational, and displays the data
* on a 4x20 LCD.  In addition to the current values, min and max values for
* temperature and wind are also stored.  In addition to displaying this data
* on the LCD, data formatted for APRS weather reporting is also sent out
* via the serial port.  This goes to a TNC (ham radio modem) that broadcasts
* the data out to whomever may be interested.  The T238+ hardware includes an
* MX614-based modem, obviating the need for a TNC.
*
* Supported Sensors
* =================
* Combo:
*   DS2760 Wind speed, direction, and temperature
* Temperature:
*   DS1920 temperature sensor (original inside Wind/Temp unit--not accurate)
*   DS19S20 temperature sensor (stand-alone sensor, much more accurate)
* Wind Speed:
*   DS2423 Counter (Input A)
* Wind Direction:
*   DS2407 switch and eight DS2401 IDs (Dallas and AAG V2 sensor)
*   DS2450 Quad A/D
* Rain gauge:
*   DS2423 counter (Input A)
* Relative Humidity:
*   DS2438 Battery monitor
* Barometric Pressure:
*   DS2438 Battery monitor
* Time:
*   DS1994 chronograph
* EMP sensor
*   DS2423 Shared with Rain Gauge (Input B)
*
*
* ==============================================================================
* ==============================================================================
*
* 1.00.0:
* Initial release.  (didn't go very far)
*
* 1.01.0:
* Bug fixes and some serious code cleanup.
*
* 1.02.0:
* This release was to support download monitor and the on-board flash
* storage of variables.  The changes to this code were fairly minor as all
* the flash programming routines are in the download monitor code.  The
* changes are: 1) Move exception vectors so they don't conflict with the
* download monitor.  2) The EEPROM routines have been removed and replaced
* with monitor calls for writes and memory accesses for reads. I/O pins
* associated with the EEPROM are now unused.
*
* 1.03.0:
* The hardware that consisted of three schematics (yukky) got consolidated
* into one schematic.  It does require swizzling the I/O.  I got reasonably
* lucky with change over to the all-in-one schematic, it only really
* affects the three bits used for the 1-wire interface.  While waiting for
* the layout to commence, am cleaning up a lot of the code, including major
* changes to the 1-wire I/O routines to implement FULL CRC checking on all
* data and to dynamically handle error conditions on the fly.  Also took
* the monolithic code and split it up into seven files.  Also implemented
* the computer mode that sends the sampled data every 5 seconds instead of
* averaged data every 5 minutes.
*
* 1.03.1:
* Changed with wind speed calibration factor to match Russes wind tunnel
* data.  We were about 8.8% high.  (See PIWIND comment)
*
* 1.03.2:
* SPECIAL CUT AFTER STARTING 1.04.0!
* Averaged temperature is wildly wrong, don't know how I missed it.
* Problem was with the division, I divided by the sample address instead of
* the contents.  Oops!
*
* 1.04.0:
* Added Error counters and Error count screen.
* MAJOR CHANGE: Moved all the data gathering routines inside the timer interrupt
*   code so that it all happens in the background.  This allows the mainline
*   code to concentrate on UI stuff.  This is a fairly huge change.  Took out
*   the timer interrupt routine and put it in a seperate file as it now gets
*   big and complicated.  See its header for the program flow.
* Routines for gathering, converting, and averaging  temperature, wind speed,
*   and direction are now consolidated into seperate files.
* Updated all comments to a consistent format, ready for distribution.
*
* 1.05.0:
* Moved use of fwdoff variable from reporting routines to gathering routines
* All wind direction reporting now done through piwdir now.
* Live wind directions support text (N,NW, etc) mode with user option
* Monitor program now has splash screen
* Added Error counter screen
* Added Current status screen date/time/sample count
* Added Today/yesterday high temp/low temp/gust screen
*
*   **** LAST VERSION FOR THE GP20 ****
*
* 1.06.0:  (never made it to release)
* Reswizled memory mapping for the GP32
* Also started putting C on "candidate" releases to keep them straight
* Added ability to NOT have my weather IDs as part of the ROM
* More graceful recovery when no IDs are programmed in Flash
* Started Peet support (not finished though)
*
* 1.06.1:
* Fixed switch settings for entering debug mode
*
* 1.07.0
* Changed copyrights to 2000 (better late than never)
* Did triple LED flash on startup for better bringup support
* Fixed error where occasionally got 1-wire no response error.  Problem was
*  when you stoped gathering data, sometimes you were in active-high mode
*  on 1-wire bus when you quit, leaving the bus driven high
* Fixed 1st transmission bug in computer format (send dashes on 1st wind speed)
* First attempt at Peet format complete!
*
* 1.07.1
* Changed Peet baud rate to 2400
* Had to increase msg buffer size to support this.
* UART messages now fired off with irqmsg routine
*
* 1.08.0 (Rain guage)
* Added Full rain guage support.
*  Includes rain guage sensor discovery
*  Stores initial value in flash for cumulative readings
*  UI screen for todays rain, yesterdays rain, and cumulative rain.
*  pxxx (rain today) format in APRS status screens
*  Rain today and cumulative rain in Peet format.
* Fixed minor bug where daily high/low temp got corrupted during sensor cal.
* DS18S20 support consisted of increasing the conversion delay from 500-750ms
*
* 1.08.1
*  Fixed rain-since-midnight flag from 'p' to 'P'.
*  Resolved mainline/interrupt variable conflicts.
*  Added monthly rain counter
*  Removed rain data in TNC string when no rain sensor present.
*  Added up to three tries for wind direction reads (should get fewer errors)
*  Added LTP-mode for cool TNCs (that I need to get)
*  Fixed typo in wind direction text.
*
* 1.08.2
*  Added APRS 'r' format (rain-past-hour)
*  Fixed display bug for 24-hour gust speed (showed 5-min gust)
*  Fixed display bug for Yesterdays Hi/Lo (had them swapped)
*  Fixed bug where rain error incremented when no rain sensor installed
*  Fixed initial-value problem with monthly rain
*  Added support for rain values greater than 9.99 inches on display
*  Added notes in wireid.asm so users can plunk in their own IDs.
*
* 1.09.0
*  Added support for optional positioned wx format.
*  Fixed bug where rain comm error not cleared
*  Fixed bug where rain data sent even if comm error
*  Better handle catastrophic errors on wind direction sensing
*  Give option of user-selection if multiple temp sensors on the bus
*  Give option of user-selection if multiple counters (wind+rain) sensors.
*
* 1.09.1
*  Added Metric suport for display (UART stuff stays the same)
*  Changed TNC-LTP baud rate from 4800 to 2400 baud (Brians fix)
*
* 1.10.0
*  Added ability to read DS1994 RTC for time on power-up
*  Added GMT-local offset to Flash so both times work on power-up
*  Added more wind speed error checking (two good readings with reset in between)
*  Check for rain guage at startup to see if removed (for the winter)
*  Finally found an app note on how to REALLY initialize a 4-bit LCD interface
*
* 1.10.1
*  Fixed Peet mode bug where APRS data got added on every 60th sample.
*  Fixed bug where rain error count didn't get reset with no guage present.
*  MADE 1.10.1S (SPECIAL) VERSION TO TEST CR-LF IN TNC-LTP MODE
*
* 1.10.2
*  Fixed a poor jump where error in wind direction cal forced re-entering
*     the temperature sensor ID again in multi-temp sensor setups.
*  Fixed a bug where averaged wind direction wasn't getting computed correctly
*     during some conditions (noisy data around due north)
*  Added support for user-settable baud rates
*
* 1.11.0
*  Found out incx doesn't increment H when X rolls from ff to 00. Fixed these areas
*  Fixed bug where baud rate setting menus blanked out the screen.
*  Added support for debug string on bad temperature and wind speed.
*  Found bug where -68 is result of NO temperature readings to average
*    Also source of 99MPH readings and strange wind direction readings.
*  Fixed a bug where wind speed never came back if interrupted (introduced in 1.10.0)
*  Added Bus voltage on status screen and debug string.
*
* 1.11.1
*  Fixed bug where some WX data gets lost at 2400 baud in APRS modes.
*  Got rid of UART "AUTO" setting, I just make it explicit now.
*  Added a diatribe on wind direction IDs and offset to wireids.asm.
*  Fixed bug that showed -68 on LCD min/max if wrong tempr sensor.
*
* 1.12.0
*  Initial humidity support (Add to setup process, Read sensor value & convert,
*    add to english and metric status screens, add to TNC and computer UART
*    strings, add to Peet string, add error counter to error screen, add error
*    counter to debug message)
*
* 1.12.1
*  AAG wind direction sensor support.
*  Now show dashes (vs 0) if no humidity sensor installed.
*  Added temporary humidity debug data to error status screen
*  Metric screen had F for temperature ID even though data was C, fixed
*  Revamped sensor calibration routines.
*   Only one menu now
*   User cycled through all sensors and asked to calibrate or not.
*   ssens routine is now only a supervisory routine, code moved to
*    sensor files now (better place for them anyway!)
*  Added humidity raw data to debug string if computed value >100%
*  Fixed bug where base rain count and found flag not reset on cal.
*  Fixed bug where Baud rate was trashed on clock setting.
*  Fixed bug where rain gauge didn't show data after cal unti reset.
*  Fixed bug where humidity got gummed up on temperatures less than 0C
*  Added PB7 output to show time in IRQ
*
* 1.12.2
*  While adding AAG support, messed up Dallas support for wind direction.
*   Forgot to turn off wind direction switch.
*
* 1.12.3
*  Fixed bug where lack of humidity sensor after one is programmed locked T238 on boot.
*  Fix where setting time messes up Peet Baud rate (I think)
*  Changed character for bad readings from dashes to dots (follows APRS standard)
*  Moved humidity averaging calculations to the humid.asm.
*  Fixed bug where wind direction cal didn't erase old values first
*  Resolved ID conflict where Rain Gauge DS2407 mistook for Wind dir 2407
*    Now ask user to resolve this if problem exists.
*  Now have the option to erase rain gauge ID if sensor not present
*  Now have the option to erase humidty ID if sensor not present
*
* 1.12.4
*   Fixed bug where local time offset didn't get saved in Flash
*   Now don't send computer humidity data if 0% (has to be an error)
*   Don't send "hxx" in TNC mode if no data in the last 5 minutes.
*   Limit Display readings to 1-100% humidity
*   Limit TNC readings to 1-99& humidity
*   User has option to have humdity temperature reading as "main" reading.
*
* 1.12.5
*   Added I/O port for radio power control (solar apps)
*     Port C bit 0 is high 10 seconds before to 10 seconds after we send data
*     The bit is high from power up until after the first transmission
*     This only makes sense for TNC and TNC-LTP modes.  No menus to support this
*   Added battery voltage sampling on port B bit 7
*     Assumes 1/3 voltage divider to work.
*     Added voltage to status screen
*     A/D now alternates between bus voltage and battery voltage every 5 seconds.
*   Moved A/D conversion routines to their own file, a2d.asm
*
* 1.13.0
*   Added option to show dewpoint instead of %RH
*   Removed Humidity "02" messages.  No need for them anymore.
*     Also removed RH data from debug screen
*   Added Language support in defines (Portuguese from Hugo is first)
*   Got rid of some IRQ timing debug code, was using PB7 twice!
*   Typo in metric status screen #2, had MPH instead of KPH
*   Fixed a bug where temperature compensation on %RH was not done correctly
*   Fixed a bug where HMTMP was always positive, no negative temperatures!
*
* 1.13.1
*   Fixed a rollover problem where November rolled to January (!!)
*   New X-Y coordinate scheme for screen updates (thanks Bill!)
*   Checked default baud rates when UART mode changes--OK.
*   Re-organized RAM into $50-$ff and >=$100 regions for better index control.
*   Moved Rain table (RANTBL) to >$100
*   Now displayed temperature and dewpoint come from user-selected temp sensor
*   Fixed bug where A/D not initialized if no rain gauge.
*   Fixed bug where dewpoint math was off by up to ~10 degrees
*   Did better job of rounding displayed temperature in C.
*   Don't show dewpoint if temperature or humidity error (only humidity before)
*
* 1.14.0
*   Bray sensor barometric pressure support imported from Byron.
*   Fixed imported bug that messed up APRS strings
*   Fixed imported bug that used mainline bug in ISR (divsor)
*   Optimized the binary-ASCII conversions in bab2m and bab2am.
*   Updated copyright notices to 2004 (forgot 2002!)
*   Fixed Humidity and BP cal routines to handle two DS2428s.
*   Added rounding to baval and bavalm in iqbar1 routine
*   Fixed math bug in metric BP calculations in bab2m
*   *** Sqirly bug in TNC BP string "went away" may come back
*   Updated main menu sequence to not show BP screen if no BP sensor
*   *** Peet format for humidity not verified!
*
* 1.14.1
*   Squeezed up/down/steady arrow for BP on to main screen.
*   Removed dedicated BP screen (trend now on main screen)
*   Now re-read DS1994 clock at local midnight (if it exists)
*   If daily stats are selected, it alternates with current data.
*
* 1.14.2
*   Fixed bad jump across UART IRQ routines in TNC messaging.
*   Fixed erroneous rain reporting when no sensor installed
*   Fixed pointer problem in computer messaging 2nd routine.
*
* 1.15.0
*   Initial T238+ support
*     Code now determines at runtime if an MX-614 is installed (mxpten)
*     Added file MX614.asm for the major MX-614 support routines.
*     Mainline code changed little, UART calls call mxqdat to queue APRS string
*     Timer interrupt calls mxtirq to run transmit state machine
*     New IRQ added on Timer2 input, calls mxsirq for bit-bit data xfer to mx614
*     NOW CONDITIONALLY RE-ENABLE INTERRUPTS INSIDE TIMER IRQ CODE FOR MX614!!!!
*     User location setup now also sets up mycall and repeater callsigns
*     New CRC16 calculator, CRC16B.  Hopefully will replace CRC16 eventually.
*   Updated copyright messages to 2004
*
* 1.15.1
*   Fixed a bug where the modem mode SSIDs were not alterable.
*   Broke out CRC routines into their own file (crcs.asm)
*   (Finally) got the parallel FCS calculation working (1/10th the CPU cycles)
*   Changed C bit in Destination SSID from 0 to 1.
*   Typo had R bits in destination ID set to 0 instead of 1
*
* 1.15.2
*   Added a debug screen to the LCD screen rotation
*     Added the Humidity raw data to the debug screen
*     Added the AAG raw data to the debug screen
*   Max wind speed reported on APRS is 10-minute per Russ (Display still 5 min)
*   Error in T238+ callsign setting routine omitted '9' availability.  OOPS!
*
* 1.15.3
*  Only show Humidity and AAG data if sensors exist.
*  Fixed bug in Max wind speed introduced in 1.15.2--gust always 0!
*
* 1.15.4
*  Updated copyrights to 2005
*  Added timeout feature to MX614 transmiter state machine.
*    Also good for not locking up if modem board is missing.
*  Fixed initialization error for modem callsigns. Was 0's, S/B spaces.
*
* 1.15.5
*  Checked wrong error flag in Peet temperature conversion. errtmp, sb errdd
*  Pretty sure I found the "Midnight lockup" bug, ck1994 can't be called in ISR
*    code, takes too long if no 1994 present.
*    Short term solution for now is to not recalibrate at midnight local.
*
* 1.15.6
*  Added code to help debug the gust problem.
*
* 1.15.7
*  Wind direction bug only when Modem mode and AAG V3 weather head
*   Bug was indirect reference instead of address (nasty!)
*  Also found error where wind direction error counter didn't increment for AAG.
*
* 1.15.8
*  Fixed Bug that messed up position when fiddling with E/W bit in position string.
*  Found a rounding error in the english to metric rain gauge code.
*  Updated Copyright strings to 2006 (1st 2006 release)
*  Changed source ID from WX1W to AP1WWX for Kenwood D7xx receivers (OKd by Rob)
*  Added 12 field to Peet mode for avg wind speed (Peet spec=1 minute, I do 5)
*   Required breaking Peet data into three groups of strings vs. 2
*  Found error where bad DP value was created when >105% RH
*   Error flag not omitted from lookup table entry
*  Fixed bug where T238+ transmitter activated in computer mode.
*
* 1.16.0 (NEVER FORMALLY RELEASED!!)
*  Added two cycling "repeater types" in addition to the user-specified one
*   WIDE 121212 is WIDE 2-1, WIDE2-2 alternating
*   WIDE 121213 is similar except the sixth is WIDE3-3
*   This mode by special request of Bob Bruninga to reduce QRM
*  Added a sensor list at the top of this file to keep me straight!
*  Added timeout checks for the humidity sensor A/D polling, may be causing lockups.
*  Found a math error on metric BP reporting (for APRS string) reported by users
*   Meant fractional value could only be .0, .1, or .2
*
* 1.16.1
*  Updated all copyright banners to 2009
*  Added support for AAG TAI603 weather head (wind speed, dir, temp)
*  Known bug is unconnected sensor yields 0C readings (not sure how to fix this yet)
*  
*
* ======================================================================
* USER-SETTABLE DEFINES FOR THE FEW COMPILE-TIME OPTIONS.
* ======================================================================
* $SET    DEBUG is for debug mode (2.4576MHz CPU clock from 4.9143MHz crystal)
* $SETNOT DEBUG is for standalone mode (8MHz CPU clock from 32MHz crystal)
$SETNOT DEBUG
*
* $SET    TESTMODE is for special test modes
* $SETNOT TESTMODE is to disable the special tes mode.
* For now it is used to do MX-615 testing
$SETNOT TESTMODE
*
* $SET WILLSIN  to load my default indoor weather IDs.
* $SET WILLSOUT to load my default outdoor weather IDs.
* $SET VIRGIN   to load no IDs (requires calibration on power-up)
* $SET USERS    to load user-specific IDs (see instructions in wireids.asm)
* ***** SET EXACTLY ONE! *****
$SETNOT WILLSIN
$SETNOT WILLSOUT
$SET    VIRGIN
$SETNOT USERS

* $SET ENGLISH   for English menus
* $SET PORTUGESE for Portugese menus (Courtesy of Hugo)
* ***** SET EXACTLY ONE! *****
$SET    ENGLISH
$SETNOT PORTUGUESE

        page
$base 10T
$Include 'gp20regs.inc'
$Include 'mymon.asm'

; APPLICATION SPECIFIC EQUATES...

init0:  equ     $38             ; LCD init byte 0 8-bit IF, 2-line, 5x7 font
init1:  equ     $28             ; LCD init byte 1 4-bit IF, 2-line, 5x7 font
init2:  equ     $14             ; LCD init byte 2 cursor shift, shift right
init3:  equ     $0c             ; LCD init byte 3 Display on, cursor off, blink off
init4:  equ     $06             ; LCD init byte 4 cursor increment, display doesn't
init5:  equ     $02             ; LCD init byte 5 Cursor at home
init6:  equ     $01             ; LCD init byte 6 Clear display

lcd1:   equ     $80             ; LCD address for line 1
lcd2:   equ     $C0             ; LCD address for line 2
lcd3:   equ     $94             ; LCD address for line 3
lcd4:   equ     $D4             ; LCD address for line 4

knone:  equ     0               ; keycode for no-key
kmenu:  equ     1               ; keycode for menu button
kup:    equ     2               ; keycode for up button
kdown:  equ     3               ; keycode for down button
ksel:   equ     4               ; keycode for select button
kwait:  equ     255             ; keycode for no-new-key
tries:  equ     16              ; try up to 16 times to get wind direction ID

$include "ram.asm"              ; the RAM table
$include "wireids.asm"          ; The 1-Wire ID tables
        page
;       This is where all the vectors that get processed by MON08 get vectored
;       to.

        org     uvtbas          ; Vector for Application timebase vector
        jmp     rstrtn          ; not used
        org     uvadc           ; Vector for Application A/D conversion Complete
        jmp     rstrtn          ; not used
        org     uvkey           ; Vector for Application Keyboard
        jmp     rstrtn          ; not used
        org     uvsctx          ; Vector for Application SCI Tx
        jmp     scitx           ; ******* USED!!! (UART data to TNC/Computer)
        org     uvscrx          ; Vector for Application SCI Rx
        jmp     rstrtn          ; not used
        org     uvscer          ; Vector for Application SCI Error
        jmp     rstrtn          ; not used
        org     uvsptx          ; Vector for Application SPI Tx
        jmp     rstrtn          ; not used
        org     uvsprx          ; Vector for Application SPI Rx
        jmp     rstrtn          ; not used
        org     uvt2ov          ; Vector for Application Timer 2 overflow
        jmp     rstrtn          ; not used
        org     uvt2c1          ; Vector for Application Timer 2 channel 1
        jmp     rstrtn          ; not used
        org     uvt2c0          ; Vector for Application Timer 2 channel 0
        jmp     rstrtn          ; not used
        org     uvt1ov          ; Vector for Application Timer 1 overflow
        jmp     rstrtn          ; not used
        org     uvt1c1          ; Vector for Application Timer 1 channel 1
        jmp     mxsirq          ; ******* USED!!! (MX-614 bit interrupt)
        org     uvt1c0          ; Vector for Application Timer 1 channel 0
        jmp     timrtn          ; ******* USED!!! (100ms general purpose timer)
        org     uvpll           ; Vector for Application PLL
        jmp     rstrtn          ; not used
        org     uvirq           ; Vector for Application external IRQ
        jmp     rstrtn          ; not used
        org     uvswi           ; Vector for Application SWI instruction
        jmp     rstrtn          ; not used
        org     uvrst           ; Vector for Application power-on-reset
        jmp     rstrtn          ; go to our general-purpose startup routine

        page
; -----------------------------------------------------------------------------
; Routine:      rstrtn
; Called By:    Power-on reset
; Calls:        wrini (write initialization byte to LCD)
;               wrcmd (write command byte to LCD)
;               wcgrm (write to LCD character generator ram)
;               wrmess (write message to LCD)
;               waits (wait in 1/10ths of a second)
;               dowx (go get data)
; Gazintas:     None
; Gazoutas:     Many (lots of initialization stuff)
; Function:     1) Set up stack to end of RAM
;               2) Set up I/O ports
;               3) Initialize LCD and show banner screens
;               4) Initialize a bazzilion variables
;               5) Fire up the timer for RTC interrupts
;               6) Do some open/short 1-wire bus checks
;               7) Go straight to the data gathering code
; -----------------------------------------------------------------------------
rstrtn: ldhx    #aprend         ; the end of application RAM
        txs                     ; make it the stack pointer
        clrh                    ; leave H 0

;       The first function is really an assembler check.  I divide RAM into
;       the $40-$ff range and then $100+ range.  If the lower range exceeds
;       $ff, this clear instruction will generate an error

        clr     ramff           ; will generate an assembler error if ra

        clr     porta           ; Port A all lows
        lda     ddra            ; get port A (can't mess with bit 0)
        and     #$0f            ; bits 7:4 floating (databus)
        ora     #$0e            ; bits 3:1 outputs (control lines)
        sta     ddra            ; bit 0 left alone (for mon08)

        mov     #$06,portb      ; 1-wire: 0 drives low, 1,2 drive high.
        mov     #$80,ddrb       ; Drive unused pin, Switches and 1-wire tristate

        mov     #$01,portc      ; PC0=radio enable, start with it enabled
        mov     #$00,portd      ; drive 0's where applicable
        mov     #$ff,ptcpue     ; pull up all pins when inputs
        jsr     mxpten          ; set up IO for MX-614 if it exists.

        bset    0,config1       ; turn off COP for now

        bset    4,portd         ; turn on the LED
        lda     #2              ; .2 second wait
        jsr     waits           ; wait it
        bclr    4,portd         ; turn off the LED
        lda     #2              ; .2 second wait
        jsr     waits           ; wait it
        bset    4,portd         ; turn on the LED
        lda     #2              ; .2 second wait
        jsr     waits           ; wait it
        bclr    4,portd         ; turn off the LED
        lda     #2              ; .2 second wait
        jsr     waits           ; wait it
        bset    4,portd         ; turn on the LED
        lda     #2              ; .2 second wait
        jsr     waits           ; wait it
        bclr    4,portd         ; turn off the LED

        lda     #init0          ; initialization byte #0 (faking 8-bit)
        jsr     wrini           ; send it
        lda     #init0          ; initialization byte #0 (faking 8-bit)
        jsr     wrini           ; send it
        lda     #init0          ; initialization byte #0 (faking 8-bit)
        jsr     wrini           ; send it
        lda     #init0          ; initialization byte #0 (faking 8-bit)
        jsr     wrini           ; send it
        lda     #init1          ; initialization byte #1 (now 4-bit)
        jsr     wrini           ; send it

        lda     #init1          ; initialization byte #1
        jsr     wrcmd           ; send it
        lda     #init2          ; initialization byte #2
        jsr     wrcmd           ; send it
        lda     #init3          ; initialization byte #3
        jsr     wrcmd           ; send it
        lda     #init4          ; initialization byte #4
        jsr     wrcmd           ; send it
        lda     #init5          ; initialization byte #5
        jsr     wrcmd           ; send it
        lda     #init6          ; initialization byte #6
        jsr     wrcmd           ; send it

        page

        lda     #00             ; special character #0 (degree symbol)
        jsr     wcgram          ; program it

        lda     #01             ; special character #1 (mm symbol)
        jsr     wcgram          ; program it

        lda     #02             ; special character #2 (m/h symbol)
        jsr     wcgram          ; program it

        lda     #03             ; special character #3 (max symbol)
        jsr     wcgram          ; program it

        lda     #04             ; special character #4 (k/h symbol)
        jsr     wcgram          ; program it

        lda     #05             ; special character #5 (up arrow)
        jsr     wcgram          ; program it

        lda     #06             ; special character #6 (down arrow)
        jsr     wcgram          ; program it

        lda     #0              ; message #0 (banner 1st page)
        jsr     wr4mes          ; write it
        lda     #40             ; two second wait
        jsr     waits           ; wait it

        lda     #2              ; message #2 (Dr. Morgan msg)
        jsr     wrmess          ; write it
        lda     #20             ; two second wait
        jsr     waits           ; wait it

; Now initialize a zillion variables.  Most of these are used by the interrupt
; service routines, but we havn't enabled any of them yet.

        clr     ruftim          ; clear SW timer MSB

        clr     curkey          ; make sure current keycode is <none>

        clr     year            ; clear year value
        lda     #1              ; month and day start with 1
        sta     month           ; clear month value
        sta     day             ; clear day value
        clr     hrs             ; clear hours value
        clr     mins            ; clear minutes value
        clr     secs            ; clear seconds value
        clr     ssecs           ; clear sub-seconds value
        clr     lhrs            ; clear hours value

        clr     rtchld          ; enable RTC counting
        lda     #$ff            ; no min/max yet special flag
        sta     temphi          ; make it so
        sta     yesthi          ; make it so
        sta     yestlo          ; make it so
        sta     wxhold          ; not 0 means don't do wx samples

        lda     firref          ; get the rain reference value
        sta     ranmth          ; make sure default is a sane value
        lda     firref+1        ; get the rain reference value
        sta     ranmth+1        ; make sure default is a sane value

        clr     errtmp          ; clear temperature error counter
        clr     errspd          ; clear wind speed error counter
        clr     errdir          ; clear wind direction error counter
        clr     errran          ; clear the rain guage error counter
        clr     errhum          ; clear humidity error counter
        clra
        sta     errbar          ; clear barometer error counter

        clr     todygs          ; clear todays max gust
        clr     todygs+1        ; MSB too
        clr     yestgs          ; clear yesterdays max gust
        clr     yestgs+1        ; MSB too

        clra                    ; 0 for memory locations >0x00ff
        sta     dbptr           ; no debug message--yet
        sta     hrroll          ; initialize hours rolled flag
        sta     dtcal           ; say we don't need calibrating
        sta     mxstat          ; MX-614 state machine state (idle)

; For the rain guage I need to see if it is installed at power-up.  This is the
; one and only sensor (so far) that someone would consider removing in the
; normal course of events.  Folk may remove it for the winter.
; Instead of using the Flash firgot flag, the rest of the software uses a RAM-
; based rirgot flag.  This flag is normally a copy of firgot unless we fail
; to read a rain guage value here at power-up.  If there was one and we fail
; to see it, I throw up a message telling the user so.
; There is a wee hole in my decision-making.  If there happens to ever be a
; legit $ffff value read from the rain sensor, I'll get a false fail.  This
; is one chance in 65536, so I'm going to cheat.

        lda     firgot          ; get the flash-based rain gauge present flag
        sta     rirgot          ; save it in RAM
        bne     dotnx           ; if no rain guage installed, no more work.
        jsr     iniran          ; otherwise go initialize the rain guage
        ldhx    rancnt          ; get the rain count
        cphx    #$ffff          ; see if all 1's (no data)
        bne     dotnx           ; if not, we got data
        mov     #$ff,rirgot     ; if $ffff, say we got no rain guage to SW
        lda     #38             ; message #38 (Rain guage missing)
        jsr     wrmess          ; write it
        lda     #20             ; two second wait
        jsr     waits           ; wait it

;       For humidty support I go in and check to see that the DS2438 EEPROM
;       values are set correctly.  If they are not, they get initialized.
;       theoretically this should only happen once in the life of the
;       sensor.
;       Also initialize the A/D converter here too.

dotnx:  jsr     humini          ; go initialize humidity sensor
        jsr     a2dini          ; initialize the A/D converter
        jsr     barini          ; initialize barometer sensor

;       Here is where we read and process the time if we have a DS1994 present.

        jsr     ck1994          ; see if DS1994 present and get time if so
        beq     notim           ; if Z set, no 1994, procede as normal

        lda     hrs             ; get GMT hours
        sub     ftoff           ; subtract offset for local time
        bpl     ckg24           ; if positive, maybe OK time
        add     #24             ; if negative, add 24 for correct time
        bra     ntok            ; and should be OK
ckg24:  cmp     #24             ; see if >24
        blt     ntok            ; if not, time is OK
        sub     #24             ; otherwise subtract 24 for correct time

ntok:   sta     lhrs            ; set local hours
        lda     #37             ; message #37 (found DS1994, set time)
        jsr     wrmess          ; write it
        lda     #20             ; two second wait
        jsr     waits           ; wait it

notim:  clr     t1ch0h          ; Clear out timer 1 chan 0 to 0000
        clr     t1ch0l          ; done

        lda     #$04            ; Timer running at 500KHz, no overflow IRQs
        sta     t1sc            ; make it so
        lda     #$58            ; T1 Ch0 to fire IRQ, no fancy stuff
        sta     t1sc0           ; make it so

        jsr     mxpten          ; If MX-614 plugged in, enable its IO

; With all the variables and timer initialized, time to allow interrupts to
; start firing.  The only enabled interrupt is the timer interrupt.
; WXHOLD is set, meaning that we will start doing the timekeeping but
; no weather gathering.

        cli                     ; enable interrupts in general
        jmp     dowx            ; go get weather data (if IDs programmed)
        page

; -----------------------------------------------------------------------------
; Routine:      menus
; Called By:    Not really callees jump back here when done
; Calls:        dowx (go get weather data)
;               stime (set the UTC and local time)
;               sopts (set various options and parameters)
;               ssens (get/calibrate temp/wind sensor addresses)
;               srain (get rain sensor address)
;               sposn (set position string)
;               wrmess (write LCD message)
; Gazintas:     None
; Gazoutas:     None
; Internals:    menun (current menu number)
; Function:     This code is in charge of the main menu loop.  Pressing the
;               menu button sequences through the various menu options.
;               Pressing select goes to that routine.  Note that this routine
;               jumps to those routines and those routines jump back here.  A
;               little ugly, but since the reset routine bypasses this routine
;               on power-up, that is why.
; -----------------------------------------------------------------------------
menus:  lda     #$ff            ; not 0
        sta     wxhold          ; stop weather gathering
        mov     #$80,ddrb       ; want high power mode off (just in case)
        clr     menun           ; start with first menu
mnlp:   lda     menun           ; get menu number
        add     #5              ; offset to message #
        jsr     wrmess          ; show it
mnwait: jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     mngo            ; if so, go do that menu
        cmp     #kmenu          ; see if menu key
        bne     mnwait          ; if not, wait for "real" key

        lda     menun           ; get menu item number
        inca                    ; increment it
        cmp     #$05            ; see if too far
        blo     mnok            ; if not, life is good
        clra                    ; otherwise start over at 0

mnok:   sta     menun           ; save it
        bra     mnlp            ; go show next menu

mngo:   lda     curkey          ; get current keypress (select)
        bne     mngo            ; wait for select to go away
        lda     menun           ; get the menu number
        beq     dowx            ; 0=do weather thing
        deca                    ; check for menu 1
        beq     lstime          ; 1=set time
        deca                    ; check for menu 2
        beq     lsopts          ; 2=set options
        deca                    ; check for menu 3
        beq     lssens          ; 3=set sensors
        deca                    ; check for menu 4
        beq     lsposn          ; 4=set position string
        bra     menus           ; if none, start over

lstime: jmp     stime           ; long jump
lsopts: jmp     sopts           ; long jump
lssens: jmp     ssens           ; long jump
lsposn: jmp     sposn           ; long jump


        page
; -----------------------------------------------------------------------------
; Routine:      dowx (do the weather thing)
; Called By:    rstrtn and menus
; Calls:        i_rst (ping the 1-wire bus for signs of life)
;               docrc8 (check Flash ID to see if valid)
;               initnc (initialize the TNC (if in TNC mode))
;               initmp,inidir,inispd (initialize weather stats)
;               showit (show current data on LCD)
;               menus (go to main menus)
; Gazintas:     newdat (new wx data available, set by timirq)
;               curkey (current key pressed)
; Gazoutas:     wxhold (wx data gathering enable)
;               cdisp (current display mode)
; Internals:    cdisp (display mode, see showit)
; Function:     This is the main loop for the wx data gathering and displaying.
;               First we check to see the bus is neither stuck low nor devoid
;               of 1-Wire slaves.  If so, we generate an error message and
;               return right away.  Next we do a quick CRC on the temperature
;               sensor ID in flash to see if anything valid is programmed
;               in there. If not, we abort too.  If all checks out,
;               next we initialize the TNC communications (if needed), then
;               go initialize all the variables needed for the three main
;               sensors.  Finally we initialize a few more irq=type variables
;               before enabling the weather sensing to start in the background.
;               Note that all the data gathering and reporting to the TNC/
;               computer is done as part of the interrupt service routine.  All
;               that needs doing here is to wait for data to be available for
;               LCD display and show it off.
;               1.04.1: Also added back the code that allows the user to
;               change display modes.  Pressing the select button sequences
;               through the display options.
; -----------------------------------------------------------------------------
dowx:
$IF TESTMODE
        jmp     mxinit          ; in test mode, just go do a TNC message
$ENDIF
        ldx     #3              ; Wanna copy four bytes to debug buffer
        clrh                    ; MSB=0
dbinil: lda     binver,x        ; get a version byte
        sta     dbbuf,x         ; store in debug buffer
        decx                    ; on to next byte
        bpl     dbinil          ; do all four bytes

        lda     #4              ; mark 4 bytes in debug buffer
        sta     dbptr           ; no debug message--yet

        brclr   0,portb,lerrl   ; If 1-wire bus is low, we got a problem
        jsr     i_rst           ; issue a 1-wire reset
        bne     lerrh           ; if no response, big problems

        clr     crc8            ; clear 8-bit CRC value
        lda     fitemp          ; get temperature ID byte #1
        jsr     docrc8          ; munge it
        lda     fitemp+1        ; get temperature ID byte #2
        jsr     docrc8          ; munge it
        lda     fitemp+2        ; get temperature ID byte #3
        jsr     docrc8          ; munge it
        lda     fitemp+3        ; get temperature ID byte #4
        jsr     docrc8          ; munge it
        lda     fitemp+4        ; get temperature ID byte #5
        jsr     docrc8          ; munge it
        lda     fitemp+5        ; get temperature ID byte #6
        jsr     docrc8          ; munge it
        lda     fitemp+6        ; get temperature ID byte #7
        jsr     docrc8          ; munge it
        lda     fitemp+7        ; get temperature ID byte #8
        jsr     docrc8          ; munge it

        lda     crc8            ; get crc result
        beq     wxit            ; if $00, a valid ID, go do weather!

lerru:  lda     #30             ; message #26 (No sensors programmed)
        jsr     wrmess          ; write it
        bra     lerrw           ; wait for them to read it

lerrl:  lda     #26             ; message #26 (Bus shorted)
        jsr     wrmess          ; write it
        bra     lerrw           ; wait for them to read it

lerrh:  lda     #27             ; message #27 (No connection)
        jsr     wrmess          ; write it

lerrw:  lda     #50             ; five seconds
        jsr     waits           ; wait it
        jmp     menus           ; and go back to menus

        page
wxit:   clra                    ; 0...
        sta     wwcnt           ; Wide counter (need this set for initnc)
        jsr     initnc          ; initialize the TNC
        jsr     initmp          ; initialize temperature stats
        jsr     inidir          ; initialize wind direction stats
        jsr     inispd          ; initialize wind speed stats
        jsr     newhum          ; initialize humidity stuff
        jsr     newbar          ; initialize barometric stuff
        jsr     iniaag          ; initialize AAG wind sensor (if there)
        clr     cumcnt          ; initialize our sample counter
        clr     irqseq          ; initialize irq sequencer
        clr     newdat          ; make sure no old data flagged

        clra                    ; 0.  Below are 16-bit, so can't use CLR
        sta     maxw2           ; clear previous max 5-min wind speed MSB
        sta     maxw2+1         ; clear previous max 5-min wind speed LSB

        clr     wxhold          ; let the sampling begin!

; ******************************************************************************
; At this point ALL interrupt service routines are enabled, so now I need to
; worry about all my shared variables.  At this point, only non-ISR variables
; can me messed with.
; ******************************************************************************

        lda     #$ff            ; no min/max yet special flag
        sta     temphi          ; make it so
        mov     #ksel,lkey      ; last key was select (for display sequencing)
        clr     cdisp           ; default display to temp & wind


        page
;       Here is the main display loop.  As of version 1.04, this loop doesn't
;       do a heck of a lot anymore as all the sampling and UART messaging has
;       been moved to the interrupt routines.  What is left is UI stuff.
;       The work now is to decide which display mode to be in and show the
;       data the user is requesting.  We only update the display in the event
;       of new data or a change of screen type.  New data shows up every five
;       seconds, and a new screen type is requested by the user pressing the
;       select button.

sloop:  lda     curkey          ; get the current key
        cbeqa   #kmenu,menusl   ; if menu, go to menus

        lda     curkey          ; get current key
        tax                     ; save it for a bit
        cmp     #ksel           ; see if Select
        bne     shosam          ; if not, no change in display
        lda     lkey            ; get previous key sample
        cmp     #knone          ; see if it was no key
        bne     shosam          ; if not, no change in display

        inc     cdisp           ; got a select key hit, increment display #
        lda     cdisp           ; get it
        cmp     #7              ; see if too big
        bne     slninc          ; if not, go decide
        clr     cdisp           ; otherwise clear display number

slninc: stx     lkey            ; save it as the last key for next time.
        lda     #0              ; 0...
        sta     cdflag          ; clear current/daily flag
        bra     slsho           ; go show new data now!

shosam: stx     lkey            ; save current key as the last key

        tst     newdat          ; test data available flag
        beq     sloop           ; if no new data, go wait some more

        clr     newdat          ; reset data-available flag
slsho:  jsr     showit          ; show the data
        bra     sloop           ; now go wait for next data.

menusl: jmp     menus           ; long jump

        page
; -----------------------------------------------------------------------------
; Routine:      showit
; Called By:    dowx
; Calls:        wrmess (write message to LCD)
;               wrcmd (write LCD command byte)
;               wrdat (write LCD data byte)
;               pitemp (convert binary to ASCII temperature)
;               piwdir (convert binary to ASCII wind direction)
;               piwind (convert binary to ASCII wind speed)
; Gazintas:     cdisp (Current display mode)
;               errtmp (error flag/counter for temperature)
;               errdir (error flag/counter for wind direction)
;               errspd (error flag/counter for wind speed)
; Gazoutas:     none
; Function:     This is the display code.  This code takes the current data
;               set and presents it on the display.  We have more data that
;               can fit on the display at once, so multiple modes are
;               supported.
;               Note that data is only displayed if it is valid data.  If
;               errors occur during the sampling, the information is blanked
;               out on the display.
; -----------------------------------------------------------------------------
showit: lda     fmmode          ; get the english/metric mode
        bne     shoeng          ; if non-0, show english (American) units
        jmp     shomet          ; if 0, do metric

shoeng: lda     cdisp           ; get the display number
        beq     show0           ; if 0, do display #0
        deca                    ; see if 1
        beq     lshow1          ; if so, do display #1
        deca                    ; see if 2
        beq     lshow2          ; if so, do display #2
        deca                    ; see if 3
        beq     lshow3          ; if so, do display #3
        deca                    ; see if 4
        beq     lshow4          ; if so, do display #4
        deca                    ; see if 5
        beq     lshow5          ; if so, do display #5
        deca                    ; see if 6
        beq     lshow6          ; if so, do display #6
        bra     show0           ; if anything else, default to 0

;       For display 1 (daily cumulative stats), we alternate between
;       Daily stats and the current data.  The value cdflag is incremented
;       every 1/10th of a second, so I check bit #6 (alternates every 6.4
;       seconds) to decide which to show.  I show stats for the first 6.4
;       seconds (it needs to be first), then alternate to the current data.
;       Cdflag get cleared each time the select button is pressed, so we always
;       start with 6.4 seconds of daily stats.

lshow1: lda     cdflag          ; get alternating counter
        and     #64             ; check bit #6
        bne     show0           ; if set, show current stat
        jmp     show1           ; otherwise show daily stats.

lshow2: jmp     show2           ; go do it.
lshow3: jmp     show3           ; go do it.
lshow4: jmp     show4           ; go do it.
lshow5: jmp     show5           ; go do it.
lshow6: jmp     show6           ; go do it.


        page
; -----------------------------------------------------------------------------
;       Display mode 0: Current weather
;
;        --------------------
;       |     Currently      |
;       |Tmp xxxoF Rain x.xx"| Current temperature and rainfall since midnight
;       |Wnd xxxo xxMPH G xx | Current wind direction, speed, 5-min gust
;       |RHxxx%DPxxxoBPxx.xx!| Relative humidity and dewpoint and Barometric pressure
;        --------------------
; -----------------------------------------------------------------------------

show0:  lda     #1              ; Big message #1 (status page 0)
        jsr     wr4mes          ; write it
        clra
        sta     mzfill          ; for this stuff, dont' zero-fill

        brset   7,errtmp,sho0a  ; if error, don't show temperature
        lda     ddtemp          ; get current temperature
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert current temp to degrees farenheit
        lda     #lcd2+4         ; Line 2, Col 4
        jsr     sho3dg          ; show temperature

sho0a:  lda     rirgot          ; see if any rain guage present
        bne     sho0b           ; if not, leave all entries blank
        brset   7,errran,sho0b  ; if sample error, skip current rain stat

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd2+15        ; todays rain location
        jsr     shoran          ; show the rain

sho0b:  brset   7,errdir,sho0c  ; if direction error, don't show data
        lda     fdmode          ; get users wind direction display mode
        jsr     pmwdir          ; convert wind direction to degrees
        lda     #lcd3+4         ; Line 3, Col 4 (wind direction)
        jsr     sho3dg          ; show it

        lda     fdmode          ; check wind direction display mode
        beq     sho0c           ; if numeric, no action required
        lda     #' '            ; if text direction erase degree symbol
        jsr     wrdat           ; do it

sho0c:  lda     errspd          ; get wind speed error counter
        and     #$c0            ; check both bits
        bne     sho0d           ; if any errors, don't show
        clra
        sta     mzfill          ; for this stuff, dont' zero-fill
        lda     winspd          ; get current wind speed MSB
        sta     macnv           ; conversion buffer
        lda     winspd+1        ; get cyrrebt wind speed LSB
        sta     macnv+1         ; conversion buffer
        jsr     pmwind          ; convert 5-sec value to MPH
        lda     #lcd3+9         ; Line 3, Col 9 (wind speed)
        jsr     sho2dg          ; show it

sho0d:  clra
        sta     mzfill          ; for this stuff, dont' zero-fill
        lda     maxws           ; get max wind speed MSB
        sta     macnv           ; conversion buffer
        lda     maxws+1         ; get max wind speed LSB
        sta     macnv+1         ; conversion buffer
        jsr     pmwind          ; convert 5-sec value to MPH
        lda     #lcd3+17        ; Line 3, Col 17 (gust speed)
        jsr     sho2dg          ; show it

sho0e:  lda     fihum           ; get humidity sensor ID
        cmp     #$26            ; see if programmed
        bne     sho0h           ; skip if no sensor programmed
        brset   7,errhum,sho0h ; if humidity error, skip too

        clra                    ; 0...
        sta     mzfill          ; No leading 0's on humidity
        clr     mbcnv           ; conversion buffer MSB
        lda     hmval           ; get current humidity value
        and     #$7f            ; clear excess flag
        sta     mbcnv+1         ; conversion buffer
        jsr     bin2am          ; convert to ASCII
        lda     #lcd4+2         ; Line 4, Col 2
        jsr     sho3dg          ; show it

sho0f:  lda     fhmtmp          ; see where temperature came from
        beq     sho0g           ; if it came from the humidity sensor, data is OK

        brset   7,errtmp,sho0h  ; if temperature error skip dewpoint

sho0g:  lda     hmdew           ; get current temperature
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert current temp to degrees farenheit
        lda     #lcd4+8         ; Line 4, Col 8
        jsr     sho3dg          ; show temperature

sho0h:  lda     fibar           ; get barometer ID
        cmp     #$26            ; see if programmed
        bne     showx0          ; skip if no bar sensor

        lda     errbar          ; get barometer error counter
        cmp     #$7f            ; see if MSB set
        bhs     showx0          ; if error ser, skip bar. sensor

        lda     baval           ; get current Barometer reading in Binary
        jsr     bab2am          ; convert to 4 digit ASCII, store in matemp(0-3)
        lda     #lcd4+14        ; Line 4, Col 14
        jsr     wrcmd           ; set it.
        lda     matemp          ; send first two digits to LCD
        jsr     wrdat           ; write it to screen
        lda     matemp+1        ; send second digit to a reg
        jsr     wrdat           ; write to screen
        lda     #'.'            ; the decimal point
        jsr     wrdat           ; send it.
        lda     matemp+2        ; Bp digit 3
        jsr     wrdat
        lda     matemp+3        ; get last digit
        jsr     wrdat

        lda     #lcd4+19        ; last character on last line
        jsr     wrcmd           ; set it
        jsr     bardel          ; see if baro. is rising, falling or steady
        jsr     wrdat           ; write message

showx0: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 1: Todays stats
;
;        --------------------
;       |    Todays Stats    |
;       |Temp Hi xxx Lo xxx  | Todays high, low temperature
;       |Max Gust xxMPH      | Todays max wind gust
;       |Rain x.xx"          | Todays rainfall
;        --------------------
; -----------------------------------------------------------------------------

show1:  lda     #12             ; Big message #12 (status page 1)
        jsr     wr4mes          ; write it
        clra
        sta     mzfill          ; for this stuff, dont' zero-fill

sho1a:  lda     temphi          ; get current hi temp
        cmp     #$ff            ; see if un-initialized
        beq     sho1b           ; if so, leave dashes
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert low temp to degrees farenheit
        lda     #lcd2+8         ; Line 2, Col 8 (low temperature)
        jsr     sho3dg          ; show it

sho1b:  lda     templo          ; get current high temp
        cmp     #$ff            ; see if un-initialized
        beq     sho1c           ; if so, leave dashes
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert high temp to degrees farenheit
        lda     #lcd2+15        ; Line 2, Col 15 (high temperature)
        jsr     sho3dg          ; show it

sho1c:  lda     todygs          ; get todays gust speed
        sta     macnv           ; conversion buffer
        lda     todygs+1        ; get current wind speed MSB
        sta     macnv+1         ; conversion buffer
        jsr     pmwind          ; convert 5-sec value to MPH
        lda     #lcd3+9         ; Line 3, Col 9
        jsr     sho2dg          ; show it

        lda     rirgot          ; see if any rain guage present
        bne     sho1d           ; if not, leave all entries blank
        brset   7,errran,sho1d  ; if sample error, skip current rain stat

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd4+5         ; todays rain location (line 4, col 5)
        jsr     shoran          ; show the rain

sho1d:  nop                     ; space waster
showx1: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 2: Yesterdays stats
;        --------------------
;       |     Yesterday      |
;       |Temp L---oF H---oF  | High and Low temp
;       |Wind Gust -- MPH    | Wind gust
;       |Rain -.--"          | Rainfall
;        --------------------
; -----------------------------------------------------------------------------

show2:  lda     #2              ; Big message #1 (status page 0)
        jsr     wr4mes          ; write it
        clra                    ; 0
        sta     mzfill          ; for this stuff, dont' zero-fill

        lda     yestlo          ; get yesterdays high temperature
        cmp     #$ff            ; see if valid
        beq     sho2sh          ; if not, skip it
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert low temp to degrees farenheit
        lda     #lcd2+6         ; Line 2, Col 6
        jsr     sho3dg          ; show it

sho2sh: lda     yesthi          ; get yesterdays low temperature
        cmp     #$ff            ; see if valid
        beq     sho2sl          ; if not, skip it
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pmtemp          ; convert low temp to degrees farenheit
        lda     #lcd2+13        ; Line 2, Col 13
        jsr     sho3dg          ; show it

sho2sl: lda     yestgs          ; get Yesterdays gust speed
        sta     macnv           ; conversion buffer
        lda     yestgs+1        ; get current wind speed MSB
        sta     macnv+1         ; conversion buffer
        jsr     pmwind          ; convert 5-sec value to MPH
        lda     #lcd3+10        ; Line 3, Col 10
        jsr     sho2dg          ; show it

        lda     rirgot          ; see if any rain guage present
        bne     showx1          ; if not, leave all entries blank

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    ranyst          ; get yesterdays rain
        sthx    mbcnv           ; store for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd4+5         ; yesterdays rain location (row 4, col 5)
        jsr     shoran          ; show the rain

showx2: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 3: All the rain stats
;        --------------------
;       |       Rain         |
;       |Today -.-- Yest -.--| Todays (so far) and yesterdays rain
;       |Month -.-- User -.--| Month (so far) and user-ref rain
;       |Hour  -.--          | Last hour
;        --------------------
; -----------------------------------------------------------------------------

show3:  lda     #3              ; big message #4 (status #3)
        jsr     wr4mes          ; write it
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall

        lda     rirgot          ; see if any rain guage present
        bne     sho3x           ; if not, leave all entries blank
        brset   7,errran,sho3cc ; if sample error, skip current rain stat

        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd2+6         ; todays rain location
        jsr     shoran          ; show the rain

sho3cc: ldhx    ranyst          ; get yesterdays rain
        sthx    mbcnv           ; store for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd2+16        ; yesterdays rain location
        jsr     shoran          ; show the rain

        brset   7,errran,sho3x  ; if sample error, skip current rain stat

;                               ; RAIN THIS MONTH
        lda     rancnt+1        ; get current rain counter LSB
        sub     ranmth+1        ; subtract reference LSB
        sta     mbcnv+1         ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     ranmth          ; subtract reference MSB
        sta     mbcnv           ; save result
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd3+6         ; cumulative rain location
        jsr     shoran          ; show the rain

;                               ; USER-SELECTABLE RAIN CUMULATION
        lda     rancnt+1        ; get current rain counter LSB
        sub     firref+1        ; subtract reference LSB
        sta     mbcnv+1         ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     firref          ; subtract reference MSB
        sta     mbcnv           ; save result
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd3+16        ; cumulative rain location
        jsr     shoran          ; show the rain

;                               ; RAIN PAST HOUR
        ldhx    ran1hr          ; get rain last hour
        sthx    mbcnv           ; save result for conversion
        jsr     pmrain          ; go convert it to ASCII
        lda     #lcd4+6         ; hour's worth rain location
        jsr     shoran          ; show the rain

sho3x:  rts                     ; all done

; -----------------------------------------------------------------------------
;       Display mode 4: Internal Status
;        --------------------
;       |     Status         |
;       |mm/dd/yy hh:mm Zulu | Date and Time (Zulu)
;       |hh:mm Lcl  Bus:-.--V| Local Time, Bus Voltage
;       |Smp --/60  Bty:--.-V| Sample count and battery voltage
;        --------------------
; -----------------------------------------------------------------------------

show4:  lda     #4              ; big message #5 (stats)
        jsr     wr4mes          ; write it
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall

        lda     month           ; get the month
        sta     mbcnv+1         ; place to convert
        clr     mbcnv           ; clear MSB
        jsr     bin2am          ; convert it
        lda     #lcd2+0         ; Line 2, Col 0 (Month)
        jsr     sho2dg          ; show two-digit number

        lda     day             ; get the day
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd2+3         ; Line 2, Col 3 (Day)
        jsr     sho2dg          ; show two-digit number

        lda     year            ; get the year
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd2+6         ; Line 2, Col 6 (Year)
        jsr     sho2dg          ; show two-digit number

        lda     hrs             ; get the Zulu hours
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd2+9         ; Line 2, Col 9
        jsr     sho2dg          ; show two-digit number

        lda     mins            ; get the Zulu minutes
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd2+12        ; Line 2, Col 12
        jsr     sho2dg          ; show two-digit number

        lda     lhrs            ; get the local hours
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd3+0         ; Line 3, Col 0
        jsr     sho2dg          ; show two-digit number

        lda     mins            ; get the Zulu minutes
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd3+3         ; Line 3, Col 3
        jsr     sho2dg          ; show two-digit number

        lda     cumcnt          ; get the current sample count
        sta     mbcnv+1         ; place to convert
        jsr     bin2am          ; convert it
        lda     #lcd4+4         ; Line 4, Col 4
        jsr     sho2dg          ; show two-digit number

        lda     a2d1w           ; get the A/D conversion value
        jsr     a2dc5v          ; convert to 500 scale

        lda     #lcd3+15        ; Line 3, Col 15
        jsr     wrcmd           ; set cursor
        lda     matemp          ; Voltage 1's
        jsr     wrdat           ; send it
        lda     #'.'            ; decimal point
        jsr     wrdat           ; send it
        lda     matemp+1        ; Voltage 1/10ths
        jsr     wrdat           ; send it
        lda     matemp+2        ; Voltage 1/100ths
        jsr     wrdat           ; send it

        lda     a2dbty           ; get the A/D conversion value
        jsr     a2dc12           ; convert to 120 scale

        lda     #lcd4+15        ; Line 4, Col 15
        jsr     wrcmd           ; set cursor
        lda     matemp          ; Voltage 10's
        jsr     wrdat           ; send it
        lda     matemp+1        ; Voltage 1s
        jsr     wrdat           ; send it
        lda     #'.'            ; decimal point
        jsr     wrdat           ; send it
        lda     matemp+2        ; Voltage 1/10ths
        jsr     wrdat           ; send it

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 5: Error Stats
;        --------------------
;       |  Error Counters    |
;       |Temp xxx Rain xxx   | Tempertaure and Rain error counters
;       |Wspd xxx Wdir xxx   | Wind speed and wind direction error counters
;       |Humdxxx  Bar  xxx   | Humidity erro
;        --------------------
; -----------------------------------------------------------------------------

show5:  lda     #5              ; message #21 (Daily stats screen)
        jsr     wr4mes          ; write it
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on error counters
        clr     mbcnv           ; make sure MSB is 0, all #'s 0-63

        lda     errtmp          ; get temperature error counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd2+5         ; Line 2, Col 5 (temperature)
        jsr     sho3dg          ; show three-digit number

        lda     errran          ; get rain guage errror counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd2+14        ; Line 2, Col 14 (rain)
        jsr     sho3dg          ; show three-digit number

        lda     errspd          ; get wind speed error counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd3+5         ; Line 3, Col 5 (speed)
        jsr     sho3dg          ; show three-digit number

        lda     errdir          ; get wind direction error counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd3+14        ; Line 3, Col 14 (wind direction)
        jsr     sho3dg          ; show three-digit number

        lda     errhum          ; get humdity sensor error counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd4+5         ; Line 4, Col 5 (humidity)
        jsr     sho3dg          ; show three-digit number

        lda     errbar          ; get barometer error counter
        and     #$3f            ; clear error flag bits
        sta     mbcnv+1         ; store for conversion
        jsr     bin2am          ; convert error counter to ASCII
        lda     #lcd4+14        ; Line 4, Col 14 (Barometer)
        jsr     sho3dg          ; show three-digit number

showx5: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 6: Debug Info
; This screen will be somewhat dynamic, showing whatever debug information
; is needed to chase down a particular problem.
;        --------------------
;       | Debug Information  |
;       |H: vaaaa abbbb tcccc| Humidity Raw data
;       |AAG: aa bb cc dd    | AAG wx data
;       |                    |
;        --------------------
; -----------------------------------------------------------------------------

show6:  lda     #17             ; message #17 Debug information screen
        jsr     wr4mes          ; write it
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on error counters

        lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     sho6a           ; if not $26, no sensor installed

        lda     hmvcc           ; get humidity VCC value LSB
        sta     mbcnv           ; save it
        lda     hmvcc+1         ; get humidity VCC value MSB
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd2+4         ; place to put it
        jsr     sho4dg          ; display it

        lda     hmvad           ; get humidity VAD value MSB
        sta     mbcnv           ; save it
        lda     hmvad+1         ; get humidity VAD value LSB
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd2+10        ; place to put it
        jsr     sho4dg          ; display it

        lda     hmtmp           ; get humidity temperature value MSB
        sta     mbcnv           ; save it
        lda     hmtmp+1         ; get humidity temperature value LSB
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd2+16        ; place to put it
        jsr     sho4dg          ; display it

        lda     fisw            ; get switch/ADC family ID
sho6a:  cmp     #$20            ; see if AAG-based setup
        bne     showx6         ; if not, exit

        clr     mbcnv           ; clear MSB (not used)
        lda     aagdbg          ; AAG wind direction byte 1
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd3+5         ; place to put it
        jsr     sho2dg          ; display it

        lda     aagdbg+1        ; AAG wind direction byte 2
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd3+8         ; place to put it
        jsr     sho2dg          ; display it

        lda     aagdbg+2        ; AAG wind direction byte 3
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd3+11        ; place to put it
        jsr     sho2dg          ; display it

        lda     aagdbg+3        ; AAG wind direction byte 4
        sta     mbcnv+1         ; save it
        jsr     bin2ax          ; convert to ASCII
        lda     #lcd3+14        ; place to put it
        jsr     sho2dg          ; display it

showx6: rts                     ; all done

        page

; -----------------------------------------------------------------------------
; Some general support routines
; sho2dg: writes two digit values from matemp to the LCD
; sho3dg: writes three digit values from matemp to the LCD
; sho4dg: writes four digit values from matemp to the LCD
; shoran: writes rain values values from matemp to the LCD
; -----------------------------------------------------------------------------

sho2dg: jsr     wrcmd           ; set it.
        lda     matemp+1        ; temperature byte 2
        jsr     wrdat           ; send it
        lda     matemp+2        ; temperature byte 3
        jsr     wrdat           ; send it
        rts                     ; all done

sho3dg: jsr     wrcmd           ; set it.
        lda     matemp          ; temperature byte 1
        jsr     wrdat           ; send it
        lda     matemp+1        ; temperature byte 2
        jsr     wrdat           ; send it
        lda     matemp+2        ; temperature byte 3
        jsr     wrdat           ; send it
        rts                     ; all done

sho4dg: jsr     wrcmd           ; set it.
        lda     matemp          ; temperature byte 1
        jsr     wrdat           ; send it
        lda     matemp+1        ; temperature byte 2
        jsr     wrdat           ; send it
        lda     matemp+2        ; temperature byte 3
        jsr     wrdat           ; send it
        lda     matemp+3        ; temperature byte 4
        jsr     wrdat           ; send it
        rts                     ; all done



;       The MSB of the last ASCII byte is set if the format should be xx.x
;       If that bit is clear, the normal x.xx format will be shown
shoran: jsr     wrcmd           ; set it.
        lda     matemp          ; rain byte 1
        jsr     wrdat           ; send it
        brclr   7,matemp+2,shonr; if flag clear, do x.xx format

        lda     matemp+1        ; rain byte 2 (xx.x format)
        jsr     wrdat           ; send it
        lda     #'.'            ; the decimal point
        jsr     wrdat           ; send it.
        bra     shorst          ; go do last character

shonr:  lda     #'.'            ; the decimal point (x.xx format)
        jsr     wrdat           ; send it.
        lda     matemp+1        ; rain byte 2
        jsr     wrdat           ; send it

shorst: lda     matemp+2        ; rain byte 3
        and     #$7f            ; mask MSB in case it was set
        jsr     wrdat           ; send it
        rts                     ; all done

;       The MSB of the last ASCII byte is set if the format should be xxx.
;       If that bit is clear, the normal xx.x format will be shown
;       METRIC version!!  Decimal is offset by one versus metric
shmran: jsr     wrcmd           ; set it.
        lda     matemp          ; rain byte 1
        jsr     wrdat           ; send it

        lda     matemp+1        ; rain byte 2
        jsr     wrdat           ; send it

        brclr   7,matemp+2,shmnr; if flag clear, do xx.x format

        lda     matemp+2        ; rain byte 2
        and     #$7f            ; mask MSB in case it was set
        jsr     wrdat           ; send it
        lda     #'.'            ; the decimal point (xxx, format)
        jsr     wrdat           ; send it.
        bra     shmrex          ; all done

shmnr:  lda     #'.'            ; the decimal point (x.xx format)
        jsr     wrdat           ; send it.
        lda     matemp+2        ; rain byte 2
        and     #$7f            ; mask MSB in case it was set
        jsr     wrdat           ; send it

shmrex: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      initnc
; Called By:    dowx
; Calls:        sbaud (set UART baud rate)
;               wrmess (write LCD message)
;               srmsg (send ROM message)
;               waits (wait in tenth seconds)
; Gazintas:     fumode (Uart mode 0=TNC, 1=computer)
;               ruftim (rough time in 1/10 second increments
; Gazoutas:     None
; Function:     This is the code that initializes the UART and (if needed) the
;               TNC.  For TNC modes, the baud rate is 4800, for computer mode
;               the baud rate is 9600, for Peet mode the baud rate is 2400.
;               For TNC direct mode we need to initialize the TNC.  The sequence
;               is as follows:
;               1) Send a <CR> Flushes any pending data in TNC buffer
;               2) Wait for buffer to clear (1 second of no data)
;               3) Send a ^C (gets to command mode if we're not there)
;               4) Send "echo off" (this keeps traffic down)
;               5) Send "monitor off" (keeps traffic down too)
;               6) Send "convers" (enter conversation mode)
;               7) Send version string (transmits this to the world)
; -----------------------------------------------------------------------------
initnc: jsr     sbaud           ; fire up the UART (regardless of mode)

        lda     fumode          ; get the UART mode 0=TNC direct, 1+=computers
        bne     initcx          ; if computer, don't initialize the TNC

        lda     #25             ; message #20 (Setting up TNC...)
        jsr     wrmess          ; write it

        ldhx    #tncmg1         ; CR message
        jsr     srmsg           ; send ROM message

wfnrx:  lda     ruftim          ; get the current .1 second counter
        add     #10             ; plus one second
        sta     temp            ; our end value

wfnrx2: lda     scs1            ; get the SCI status
        ldx     scdr            ; dummy read of the data in case

        and     #$20            ; get data ready bit
        bne     wfnrx           ; if data, restart 1-second timer
        lda     ruftim          ; get the current rough time
        cmp     temp            ; see if past 1 second
        bne     wfnrx2          ; if not, go poll some more

        ldhx    #tncmg2         ; ^C message
        jsr     srmsg           ; send ROM message

        lda     #10             ; one-second wait
        jsr     waits           ; wait that long

        ldhx    #tncmg3         ; echo off message MSB
        jsr     srmsg           ; send ROM message

        ldhx    #tncmg4         ; monitor off message
        jsr     srmsg           ; send ROM message

        ldhx    #tncmg5         ; convers message MSB
        jsr     srmsg           ; send ROM message

        ldhx    #tncmg6         ; Hello World message
        jsr     srmsg           ; send ROM message

        lda     #19             ; message #19 (one moment please)
        jsr     wrmess          ; write it

initcx: lda     fumode          ; get mode
        cmp     #4              ; see if modem mode
        bne     initxx          ; if not, exit

        ldhx    #tncmg7         ; Hello World message with T238+ flag
        jsr     mxmsg           ; send it over the airwaves if in modem mode.

initxx: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      mthmax
; Called By:    timirq, stime
; Calls:        None.
; Gazintas:     month (the current month)
;               year (the current year)
;               mthtbl (table of days in each month)
; Gazoutas:     A: the number of days in the month
; Function:     This routine figgers out the number of days in the current
;               month. For most months, this is a simple lookup of the
;               current month.  For February I need to keep track of leap
;               years.  Fortunately, Feb 2000 is a normal leapyear, so I don't
;               have to make special provisions. Good 'til 2199 I think.
; WARNING!!     This code is called by two mainline routines (stime and ck1994)
;               and an interrupt service routine.  The code is NOT
;               re-entrant as it uses RAM locations for inputs.  In this case
;               it does not matter as interrupts are disabled while the time
;               is being set.
; -----------------------------------------------------------------------------
mthmax: lda     month           ; get the current month
        cmp     #2              ; see if February
        bne     mnmaxm          ; if not Feb, normal processing
        lda     year            ; if feb, get year
        and     #$03            ; look at two LSBs of year only
        bne     mnmaxm          ; if not 00 (4,8,12...) regular year
        lda     #29             ; Only thing left are leapyears
        bra     mnmaxx          ; go figger it

mnmaxm: ldx     month           ; put month in X as index
        clrh                    ; make sure H is 0
        decx                    ; convert 1-12 to 0-11
        lda     mthtbl,x        ; get days in the month

mnmaxx: rts                     ; A has month max value

$include "metric.asm"           ; metric display routines.
$include "timirq.asm"           ; Timer tick interrupt service routines

$include "tempr.asm"            ; Temperature reading and conversion routines
$include "wspeed.asm"           ; Wind speed reading and conversion routines
$include "wdir.asm"             ; Wind direction reading and conversion routines
$include "rain.asm"             ; Rain reading and conversion routines.
$include "humid.asm"            ; Humidity reading and conversion routines.
$include "baro.asm"             ; Barometric pressure reading and conversion routines

$include "onewire.asm"          ; One-wire drivers
$include "crcs.asm"             ; CRC routines
$include "setup.asm"            ; Setup and configuration routines
$include "uart.asm"             ; UART drivers
$include "peet.asm"             ; Peet-specific code
$include "lcd4.asm"             ; 4-bit interface LCD drivers
$include "util.asm"             ; General utilities
$include "timelog.asm"          ; DS1994 time/data logging routines
$include "a2d.asm"              ; A/D converter routines
$include "tai603.asm"           ; TAI603 weather head io routines
$include "mx614.asm"            ; MX-614 support
$include "tables.asm"           ; English data tables
$include "irqs.asm"             ; Interrupt documentation

$IF ENGLISH
$include "english.asm"          ; English data tables
$ENDIF

$IF PORTUGUESE
$include "portgse.asm"          ; Portuguese data tables
die ; Not working yet! (wrong character set & status page updates)
$ENDIF

