        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         irqs.asm
; Function:     This is a documentation file only.  It contains no code.
;               This files purpose is to describe the interrupt structure
;               of the T238 code.  It is getting complicated, so I need a
;               seperate file desribing how it works.
; -----------------------------------------------------------------------------
; There are three active interrupts:
; 1) Timer0 (main sequencer for all data gathering, IRQ every 100ms)
; 2) UART (IRQ-driven data out.  No data inputs yet)
; 3) Timer1 (used as external interrupt input for MX-614 data out)
; -----------------------------------------------------------------------------
; Once initialization is complete, interrupts in general remain enabled all
; the time.  They are turned off on occasion by the mainline code for short
; periods of time.
; The main times they are turned off are during flash data area updates during
; the setup routines.  The reason for this is that the sequence of flash
; accesses during flash updates cannot be interrupted by an interrupt.
; In pmrain and pnrain I disable interrups so I can use the divu routine.
; In rddir I need to read the wind direction outside the IRQs, so I disable
;  interrupts to prevent conflicts.
; -----------------------------------------------------------------------------
; The timer0 interrupt is in charge of the main 100ms periodic interrupt.  This
; interrupt runs almost all of the backround tasks including the main sequencer
; that in charge of all data gathering, data conversion, data formatting into
; APRS/Peet, and queuing up for the UART.  This IRQ is also responsible for
; maintaining the real-time clock and button scanning.
; The file handling this is timirq.asm.
; -----------------------------------------------------------------------------
; The UART interrupt is for sending out the UART data.  Data is queued up by
; either the timer routines or mainline code depending.  To queue up data, the
; caller calls irqmsg.  This sets a pointer to the first byte of (msg) and
; enable the UART interrupt to send the message as a background interrupt-
; driven task.
; The file handling this is uart.asm.
; -----------------------------------------------------------------------------
; The Timer1 interrupt is in charge of sending bit-data to the MX-614 modem.
; The MX-614 expects to be fed data at the bit rate (not byte rate) in
; real time.  The /ready line from the MX-614 is fed to the timer1 input to
; generate a "ready" interrupt.  The CPU must respond within about 500us.
; Timer0-based interrupts can take much longer than 500us and contains code
; that is not interruptable.  Hence we have a bit of a problem...
; -----------------------------------------------------------------------------
; The hard real-time requirements of the MX-614 mean that we need to have some
; form of nested interrupt support.  Within the timer-based routines are the
; one-wire commands that have their own hard real-time requirements, so simply
; re-enabling interrupts isn't feasible.  The hard real-time requirements are
; limited to the actual bit-banging of the one-wire data line.  The solution
; is to conditionally re-enable interrupts early in the timer service routine.
; When we get to the routines to bit-bang the one-wire bits, we disable
; interrupts again and then conditionally re-enable them again between bits.
; The conditions on re-enabling interrupts is that we have pending data for
; the MX-614.
; -----------------------------------------------------------------------------
