        page
******************************************************
** Copyright 2003 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         rain.asm
; Function:     These are the rain guage support routines.
;                 EMP sensor shares the same DS2423, so cal routine shared.
; Routines:     iniran: Initialize rain measurements
;               iqrain: Read rain data
;               ranstt: Do rain stats
;               pirain: Convert rain count to ASCII
;               pmrain: Convert rain count to ASCII (mainline)
;               pnrain: Convert rain count to ASCII (metric mainline)
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      calran
; Called By:    ssens
; Calls:        i_rst (do a 1-wire bus reset
;               wrmess (write message to LCD)
;               waits (wait in 1/10th second increments)
;               wfnkey (wait for no key pressed)
;               appera (application-called flash erase)
;               apppgm (application-called flash program)
;               isrch (search for a wind directio ID)
; Gazintas:     None
; Gazoutas:     firain (current rain sensor address)
; Function:     finds and stores the rain sensor ID.  This gets a bit tricky
;               as the same ID is used for the wind speed counter.
; -----------------------------------------------------------------------------

;       Before we get started, we make sure the user has the rain gauge
;       installed.  Several of the previous cal routines ask that the
;       rain gauge be removed, so I've been asked to put in a reminder
;       to install the rain gauge if it is removed.

calran: lda     #47             ; re-insert sensor message
        jsr     wrmess          ; write it

calrwq: jsr     wfnkey          ; wait for next keypress
        cmp     #ksel           ; see if select button (continue)
        bne     calrwq          ; if not, wait till it is

;       First go see if there is a valid wind sensor ID.  My scheme requires
;       that that ID gets progarmmed first and then any additional sensor
;       must by definition be the rain guage.  If no sensor is programmed,
;       then we tell them to go run that calibration first.

        clr     crc8            ; clear 8-bit CRC value
        lda     ficntr          ; get temperature ID byte #1
        jsr     docrc8          ; munge it
        lda     ficntr+1        ; get temperature ID byte #2
        jsr     docrc8          ; munge it
        lda     ficntr+2        ; get temperature ID byte #3
        jsr     docrc8          ; munge it
        lda     ficntr+3        ; get temperature ID byte #4
        jsr     docrc8          ; munge it
        lda     ficntr+4        ; get temperature ID byte #5
        jsr     docrc8          ; munge it
        lda     ficntr+5        ; get temperature ID byte #6
        jsr     docrc8          ; munge it
        lda     ficntr+6        ; get temperature ID byte #7
        jsr     docrc8          ; munge it
        lda     ficntr+7        ; get temperature ID byte #8
        jsr     docrc8          ; munge it

        lda     crc8            ; get crc result
        beq     srcont          ; if $00, a valid ID, go get ID

        lda     #33             ; message #33 (ask them to run other cal first)
        jsr     wrmess          ; write it
        lda     #40             ; four second wait
        jsr     waits           ; wait it
        rts                     ; and return

;       Next we go find some IDs.  There should be exactly two out there with
;       one matching the wind speed counter and the other being the rain guage.
;       I go find one ID first and compare it to the wind speed ID. If they
;       don't match, I'm all set.  If they do match, I go find the other ID.
;       if the second ID also matches, that means there is only one ID out
;       there and the rain guage is not installed.

srcont: clr     flg12           ; lower ID first...
        lda     #$1d            ; wind speed sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; make sure we got something
        beq     srnsns          ; if nothing, say so

        clrh                    ; 0...
        ldx     #8              ; wanna compare 8 bytes
srcpl1: decx                    ; want 0 at end
        lda     mndray,x        ; get byte from just-read ID table
        cmp     ficntr,x        ; compare to wind counter ID table
        bne     srgid           ; if not same, we got a match
        decx                    ; otherwise decrement counter
        bne     srcpl1          ; loop till all done

        inc     flg12           ; see if another counter ID
        lda     #$1d            ; wind speed sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; make sure we got something
        beq     srnsns          ; if nothing, say so

        clrh                    ; 0...
        ldx     #8              ; wanna compare 8 bytes
srcpl2: decx                    ; want 0 at end
        lda     mndray,x        ; get byte from just-read ID table
        cmp     ficntr,x        ; compare to wind counter ID table
        bne     srgid           ; if not same, we got a match
        decx                    ; otherwise decrement counter
        bne     srcpl2          ; loop till all done

srnsns: lda     #34             ; message #34 (No sensor found)
        jsr     wrmess          ; write it
srnsq:  jsr     wfnkey          ; wait for next keypress
        cmp     #kup            ; Up means erase IDs
        beq     srnser          ; if so, go erase them
        cmp     #kdown          ; Down means ignore
        beq     srexit          ; if so, leave w/o erasing
        bra     srnsq           ; if neither, wait for valid key

srnser: sei                     ; interrupts off
        ldhx    #fpage3         ; page to erase
        jsr     appera          ; go earse it
        cli                     ; interrupts back on
        bra     srexit          ; and exit

;       We got here if we found a counter ID that was not the wind speed
;       counter.  All it can be is the rain guage counter.  Store it.
;       The sensor ID is already in mndray.

srgid:  sei                     ; interrupts off
        ldhx    #fpage3         ; page to erase
        jsr     appera          ; go earse it
        ldhx    #firain         ; rain guage ID address
        jsr     apppgm          ; go program it

        clr     mndray          ; Got-a-rain-guage flag
        clr     rirgot          ; flag RAM flag too

        jsr     iqrgot          ; go get the current rain counter
        ldhx    rancnt          ; get the counter value
        sthx    mndray+1        ; save it for writing to flash

        lda     #$ff            ; clear out the rest
        sta     mndray+3        ; clear the rest to unprogrammed
        sta     mndray+4        ; clear the rest to unprogrammed
        sta     mndray+5        ; clear the rest to unprogrammed
        sta     mndray+6        ; clear the rest to unprogrammed
        sta     mndray+7        ; clear the rest to unprogrammed
        ldhx    #firgot         ; got-it flag
        jsr     apppgm          ; go program it

        cli                     ; interrupts back on

        jsr     iniran          ; go initialize counters

        lda     #35             ; message #35 (Success)
        jsr     wrmess          ; write it
        lda     #40             ; four second wait
        jsr     waits           ; wait it

srexit: rts                     ; all done


; -----------------------------------------------------------------------------
; Routine:      iniran
; Called By:    TBD
; Calls:        iqrain (read current rain counter)
; Gazintas:     None
; Gazoutas:     ranref: set to current rain guage count.
;               errran: Error flag cleared.
; Function:     This is a bit different than the other initialization routines
;               as the sensor is a bit different.  The rain sensor has a
;               battery in it, so initializing only needs to be done at
;               power up, not each time we enter the data logging mode.
; -----------------------------------------------------------------------------
iniran: bsr     iqrain          ; go read the current rain counter
        ldhx    rancnt          ; get the result
        sthx    ranref          ; save as a reference
        sthx    ranmth          ; initialize monthly reference too

        ldhx    #rantbl         ; point to first byte in rain table
inirlp: mov     rancnt,X+       ; move MSB
        mov     rancnt+1,X+     ; Move LSB
        cphx    #rantbl+24      ; see if at end
        bne     inirlp          ; if not, go around again

        clr     errran          ; clear the error counter
        clr     rantdy          ; clear todays rain MSB
        clr     rantdy+1        ; LSB too
        clr     ranyst          ; clear yesterdays rain MSB
        clr     ranyst+1        ; LSB too
        clr     ran1hr          ; rain last hour is 0
        clr     ran1hr+1        ; LSB too
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqrain, iqrgot
; Called By:    timrtn, iniran, srain
; Calls:        sndadr (send 1-wire address)
;               i_writ (write 1-wire byte with CRC)
;               i_read (read 1-wire byte with CRC)
;               crcini (initialize CRC engin)
;               crcchk (check CRC result)
; Gazintas:     None
; Gazoutas:     rancnt (current rain guage counter value)
;               errran (rain guage error counter)
; Function:     This is the routine to read the rain guage counter.  That
;               is all it does for now as the read data has multiple uses.
;
;               NOTE:  srain jumps straight to the iqrgot address because it
;               already programmed the ID, but wants to get the count before
;               it rirgot flag has been set.  Since it knows it just
;               programmed the ID, it knows it is safe to go read data.
; -----------------------------------------------------------------------------
iqrain: lda     rirgot          ; get our got-it flag
        beq     iqrgot          ; if so, go read data
        clr     rancnt          ; otherwise default to 0
        clr     rancnt+1        ; LSB too
        bra     iqrex           ; all done!

iqrgot: bclr    7,errran        ; clear error flag (if set)
        ldhx    #firain         ; address for rain counter
        jsr     sndadr          ; send it off
        jsr     crcini          ; reset the 16-bit CRC here
        lda     #$a5            ; read memory+counter command
        jsr     i_writ          ; send it
        lda     #$df            ; memory address LSB
        jsr     i_writ          ; send it
        lda     #$01            ; memory address MSB
        jsr     i_writ          ; send it
        jsr     i_read          ; read back memory byte (& toss)
        jsr     i_read          ; counter LSB
        sta     rancnt+1        ; store it
        jsr     i_read          ; counter byte 2
        sta     rancnt+0        ; store it
        jsr     i_read          ; counter byte 3 (ignored)
        jsr     i_read          ; counter byte 4 (ignored)
        jsr     i_read          ; dummy byte 1 (ignored)
        jsr     i_read          ; dummy byte 2 (ignored)
        jsr     i_read          ; dummy byte 3 (ignored)
        jsr     i_read          ; dummy byte 4 (ignored)
        jsr     i_read          ; CRC byte 1
        jsr     i_read          ; CRC byte 2
        jsr     crcchk          ; see if we get the magic answer
        beq     iqrex           ; if CRC good, all done

        lda     errran          ; get rain guage error counter
        inca                    ; increment wind speed error counter value
        ora     #$80            ; set error flag
        sta     errran          ; save counter and flag

iqrex:  rts                     ; all done sampling

        page
; -----------------------------------------------------------------------------
; Routine:      ranstt
; Called By:    timrtn
; Calls:        None
; Gazintas:     ranref (rain guage 24-hour reference)
;               rancnt (current rain guage counter value)
;               errran (error flags/counter)
; Gazoutas:     rantdy (rain so far today)
; Function:     This is the statistics gathering code.  The only stats I
;               curretly do is to figure out the rain so far today.
; -----------------------------------------------------------------------------
ranstt: clr     rantdy          ; Clear count MSB
        clr     rantdy+1        ; Clear count LSB
        brset   7,errran,rranex ; if read error, leave with 0 rain

        lda     rancnt+1        ; get current rain count LSB
        sub     ranref+1        ; subtract midnight LSB
        sta     rantdy+1        ; store difference LSB
        lda     rancnt          ; get current rain count MSB
        sbc     ranref          ; subtract midnight MSB
        sta     rantdy          ; store difference MSB

        lda     rancnt+1        ; get current rain counter LSB
        sub     rantbl+23       ; subtract hour ago LSB
        sta     ran1hr+1        ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     rantbl+22       ; subtract hour ago MSB
        sta     ran1hr          ; save result

rranex: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      pirain
; Called By:    tncmsg,cmpmsg
; Calls:        bin2as (convert binary to ASCII)
; Gazintas:     divdnd (rain count value (1/100ths of an inch))
; Gazoutas:     atemp (ASCII rain count)
; Function:     This is the routine to convert rain ticks to rain inches.
;               Thanks to the ticks being 1/100ths of an inch and the
;               reporting mechinism being 1/100ths of an inch, this conversion
;               devolves into a binary to ASCII conversion routine.
; -----------------------------------------------------------------------------
pirain: jsr     bin2as          ; convert
        rts                     ; done (really!)

        page
; -----------------------------------------------------------------------------
; Routine:      newran
; Called By:    timrtn
; Calls:        none
; Gazintas:     rantbl (rain entry table)
; Gazoutas:     rantbl (rain entry table)
;               ran1hr (ran past hour)
; Function:     This is the routine to shift the rain table.  This table gets
;               shifted every 5 minutes so the last entry is the rain count
;               roughly an hour ago.  This routine gets executed every five
;               minutes to shift out the oldest entry and shift in the current
;               rain count.
;               See rantbl in ram.asm for description.
;               In the event the current rain sample is bad, I essentially
;               just copy the sample from 5 minutes ago into the current-sample
;               location.  Since I just shifted the 5-minute-old data from the
;               most recent location, this just means I don't copy the new
;               data to the most recent location.
;               Since I need the rain the last hour in several places, I
;               compute it here and save it for other routines.
; -----------------------------------------------------------------------------
newran: clrh                    ; inxex MSB=0
        ldx     #21             ; source offset
newrlp: lda     rantbl,x        ; get souce byte
        sta     rantbl+2,x      ; write two bytes later
        decx                    ; decrement pointer
        bpl     newrlp          ; do all old bytes

        brset   7,errran,newrer ; if read error, special processing
        lda     rancnt          ; get rain count MSB
        sta     rantbl          ; save it
        lda     rancnt+1        ; get rain count LSB
        sta     rantbl+1        ; save it

newrer: ; if error, just leave last 5-minute sample alone (see header)
        rts                     ; done
        page
; -----------------------------------------------------------------------------
; Routine:      pmrain
; Called By:    showit
; Calls:        bin2am (convert binary to ASCII)
; Gazintas:     mbcnv (rain count value (1/100ths of an inch))
; Gazoutas:     matemp (ASCII rain count)
; Function:     This is the mainline version of pirain (no variable conflicts)
;               This is different as unlike the strings that go out the UART
;               (pirain), the display routines can show rain amounts greater
;               than 9.99 inches.  This routine plunks out a normal 3-digit
;               ASCII value for rain amounts less than 9.99 inches.  For
;               amounts greater than 9.99 inches, the value is divided by 10
;               and that value is put in the string with the MSB of the last
;               digit set high as a flag that the value is now in decimal
;               tenths of an inch.  No provision is made for values greater
;               than 99.9 inches, sorry.
; -----------------------------------------------------------------------------
pmrain: ldhx    mbcnv           ; get rain count
        cphx    #999            ; see if less than 9.99 inches
        bls     pmrnrm          ; if so, "normal" processing

        sei                     ; stop interrupts so I can use divu
        sthx    divdnd          ; rain count is divident
        mov     #10,divsor      ; wanna divide by 10
        jsr     divu            ; go divde it
        ldhx    quotnt          ; get answer
        lda     remndr          ; get remainder
        cli                     ; interrupts back available now
        cmp     #5              ; see if we need to round up
        blo     pmrnrd          ; if not, skip rounding
        aix     #1              ; if so, round up one
pmrnrd: sthx    mbcnv           ; put divided number back in place to convert
        jsr     bin2am          ; go convert to ASCII
        bset    7,matemp+2      ; set the tenths flag
        rts                     ; all done

pmrnrm: jsr     bin2am          ; convert
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      pnrain
; Called By:    showit
; Calls:        bin2am (convert binary to ASCII)
; Gazintas:     mbcnv (rain count value (1/100ths of an inch))
; Gazoutas:     matemp (ASCII rain count in mm)
; Function:     This is the metric mainline version of pirain (no variable
;               conflicts).
;               To convert 1/100ths of an inch to 1/10ths of a millimenter,
;               I need to multiply by 2.54 or $A3 hex in 2.6 format.
; -----------------------------------------------------------------------------
pnrain: sei                     ; stop interrupts
        ldhx    mbcnv           ; get 1/100ths of an inch
        sthx    divdnd          ; number to multiply
        lda     #$a3            ; 2.54
        sta     divsor          ; multiplier
        jsr     mul816          ; multiply (result is 18:6)

        lsl     quotnt+1        ; shift everything left two bits for 16:8
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB
        lsl     quotnt+1        ; shift everything left two bits for 16:8
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB

        ldhx    remndr          ; load H with MSB (x is junk)
        ldx     quotnt          ; get correct LSB in X
        sthx    mbcnv           ; save result (1/10ths of a mm)
        cli                     ; interrupts back on

        ldhx    mbcnv           ; get rain count
        cphx    #999            ; see if less than 99.9 mm
        bls     pnrnrm          ; if so, "normal" processing

        sei                     ; stop interrupts so I can use divu
        sthx    divdnd          ; rain count is divident
        mov     #10,divsor      ; wanna divide by 10
        jsr     divu            ; go divde it
        ldhx    quotnt          ; get answer
        lda     remndr          ; get remainder
        cli                     ; interrupts back available now
        cmp     #5              ; see if we need to round up
        blo     pnrnrd          ; if not, skip rounding
        aix     #1              ; if so, round up one
pnrnrd: sthx    mbcnv           ; put divided number back in place to convert
        jsr     bin2am          ; go convert to ASCII
        bset    7,matemp+2      ; set the tenths flag
        rts                     ; all done

pnrnrm: jsr     bin2am          ; convert
        rts                     ; done

